@@ -124,13 +124,53 @@ gfx12_init_compute_preamble_state(const struct ac_preamble_state *state,
    ac_pm4_set_reg(pm4, R_00B9F4_COMPUTE_DISPATCH_TUNNEL, 0);
 }
 
+static void
+cdna_init_compute_preamble_state(const struct ac_preamble_state *state,
+                                  struct ac_pm4_state *pm4)
+{
+   const struct radeon_info *info = pm4->info;
+   const uint32_t compute_cu_en = S_00B858_SH0_CU_EN(info->spi_cu_en) |
+                                  S_00B858_SH1_CU_EN(info->spi_cu_en);
+
+   /* Compute registers. */
+   /* Disable profiling on compute chips. */
+   ac_pm4_set_reg(pm4, R_00B82C_COMPUTE_PERFCOUNT_ENABLE, 0);
+   ac_pm4_set_reg(pm4, R_00B834_COMPUTE_PGM_HI, S_00B834_DATA(info->address32_hi >> 8));
+   ac_pm4_set_reg(pm4, R_00B858_COMPUTE_STATIC_THREAD_MGMT_SE0, compute_cu_en);
+   ac_pm4_set_reg(pm4, R_00B85C_COMPUTE_STATIC_THREAD_MGMT_SE1, compute_cu_en);
+   ac_pm4_set_reg(pm4, R_00B864_COMPUTE_STATIC_THREAD_MGMT_SE2, compute_cu_en);
+   ac_pm4_set_reg(pm4, R_00B868_COMPUTE_STATIC_THREAD_MGMT_SE3, compute_cu_en);
+   ac_pm4_set_reg(pm4, R_00B878_COMPUTE_THREAD_TRACE_ENABLE, 0);
+
+   if (info->family >= CHIP_GFX940) {
+      ac_pm4_set_reg(pm4, R_00B89C_COMPUTE_TG_CHUNK_SIZE, 0);
+      ac_pm4_set_reg(pm4, R_00B8B4_COMPUTE_PGM_RSRC3, 0);
+   } else {
+      ac_pm4_set_reg(pm4, R_00B894_COMPUTE_STATIC_THREAD_MGMT_SE4, compute_cu_en);
+      ac_pm4_set_reg(pm4, R_00B898_COMPUTE_STATIC_THREAD_MGMT_SE5, compute_cu_en);
+      ac_pm4_set_reg(pm4, R_00B89C_COMPUTE_STATIC_THREAD_MGMT_SE6, compute_cu_en);
+      ac_pm4_set_reg(pm4, R_00B8A0_COMPUTE_STATIC_THREAD_MGMT_SE7, compute_cu_en);
+   }
+
+   ac_pm4_set_reg(pm4, R_0301EC_CP_COHER_START_DELAY, 0);
+
+   /* Set the pointer to border colors. Only MI100 supports border colors. */
+   if (info->family == CHIP_MI100) {
+      ac_pm4_set_reg(pm4, R_030E00_TA_CS_BC_BASE_ADDR, state->border_color_va >> 8);
+      ac_pm4_set_reg(pm4, R_030E04_TA_CS_BC_BASE_ADDR_HI,
+                     S_030E04_ADDRESS(state->border_color_va >> 40));
+   }
+}
+
 void
 ac_init_compute_preamble_state(const struct ac_preamble_state *state,
                                struct ac_pm4_state *pm4)
 {
    const struct radeon_info *info = pm4->info;
 
-   if (info->gfx_level >= GFX12) {
+   if (!info->has_graphics) {
+      cdna_init_compute_preamble_state(state, pm4);
+   } else if (info->gfx_level >= GFX12) {
       gfx12_init_compute_preamble_state(state, pm4);
    } else if (info->gfx_level >= GFX10) {
       gfx10_init_compute_preamble_state(state, pm4);
@@ -453,6 +493,29 @@ gfx10_init_graphics_preamble_state(const struct ac_preamble_state *state,
                   S_00B524_MEM_BASE(info->address32_hi >> 8));
 
    /* Context registers. */
+   if (info->gfx_level >= GFX11) {
+      /* These are set by CLEAR_STATE on gfx10. We don't use CLEAR_STATE on gfx11. */
+      ac_pm4_set_reg(pm4, R_028030_PA_SC_SCREEN_SCISSOR_TL, 0);
+      ac_pm4_set_reg(pm4, R_028240_PA_SC_GENERIC_SCISSOR_TL, S_028240_WINDOW_OFFSET_DISABLE(1));
+      ac_pm4_set_reg(pm4, R_028244_PA_SC_GENERIC_SCISSOR_BR, S_028244_BR_X(16384) | S_028244_BR_Y(16384));
+      ac_pm4_set_reg(pm4, R_02835C_PA_SC_TILE_STEERING_OVERRIDE, info->pa_sc_tile_steering_override);
+      ac_pm4_set_reg(pm4, R_0283E4_PA_SC_VRS_RATE_CACHE_CNTL, 0);
+      ac_pm4_set_reg(pm4, R_028428_CB_COVERAGE_OUT_CONTROL, 0);
+      ac_pm4_set_reg(pm4, R_0286DC_SPI_BARYC_SSAA_CNTL, 0);
+      ac_pm4_set_reg(pm4, R_0287D4_PA_CL_POINT_X_RAD, 0);
+      ac_pm4_set_reg(pm4, R_0287D8_PA_CL_POINT_Y_RAD, 0);
+      ac_pm4_set_reg(pm4, R_0287DC_PA_CL_POINT_SIZE, 0);
+      ac_pm4_set_reg(pm4, R_0287E0_PA_CL_POINT_CULL_RAD, 0);
+      ac_pm4_set_reg(pm4, R_028820_PA_CL_NANINF_CNTL, 0);
+      ac_pm4_set_reg(pm4, R_028824_PA_SU_LINE_STIPPLE_CNTL, 0);
+      ac_pm4_set_reg(pm4, R_02883C_PA_SU_OVER_RASTERIZATION_CNTL, 0);
+      ac_pm4_set_reg(pm4, R_028840_PA_STEREO_CNTL, 0);
+      ac_pm4_set_reg(pm4, R_028A50_VGT_ENHANCE, 0);
+      ac_pm4_set_reg(pm4, R_028A8C_VGT_PRIMITIVEID_RESET, 0);
+      ac_pm4_set_reg(pm4, R_028AB4_VGT_REUSE_OFF, 0);
+      ac_pm4_set_reg(pm4, R_028C40_PA_SC_SHADER_CONTROL, 0);
+   }
+
    if (info->gfx_level < GFX11) {
       ac_pm4_set_reg(pm4, R_028038_DB_DFSM_CONTROL, S_028038_PUNCHOUT_MODE(V_028038_FORCE_OFF));
    }
@@ -491,6 +554,8 @@ gfx10_init_graphics_preamble_state(const struct ac_preamble_state *state,
                   S_028830_SMALL_PRIM_FILTER_ENABLE(1));
 
    ac_pm4_set_reg(pm4, R_028A18_VGT_HOS_MAX_TESS_LEVEL, fui(64));
+   if (info->gfx_level >= GFX11) /* cleared by CLEAR_STATE on gfx10 */
+      ac_pm4_set_reg(pm4, R_028A1C_VGT_HOS_MIN_TESS_LEVEL, fui(0));
    ac_pm4_set_reg(pm4, R_028AAC_VGT_ESGS_RING_ITEMSIZE, 1);
    ac_pm4_set_reg(pm4, R_028B50_VGT_TESS_DISTRIBUTION,
                   info->gfx_level >= GFX11 ?
@@ -511,10 +576,10 @@ gfx10_init_graphics_preamble_state(const struct ac_preamble_state *state,
    ac_pm4_set_reg(pm4, R_028C48_PA_SC_BINNER_CNTL_1,
                   S_028C48_MAX_ALLOC_COUNT(info->pbb_max_alloc_count - gfx10_one) |
                   S_028C48_MAX_PRIM_PER_BATCH(1023));
-
-   if (info->gfx_level >= GFX11_5)
+   if (info->gfx_level >= GFX11) {
       ac_pm4_set_reg(pm4, R_028C54_PA_SC_BINNER_CNTL_2,
-                     S_028C54_ENABLE_PING_PONG_BIN_ORDER(1));
+                     S_028C54_ENABLE_PING_PONG_BIN_ORDER(info->gfx_level >= GFX11_5));
+   }
 
    /* Break up a pixel wave if it contains deallocs for more than
     * half the parameter cache.
@@ -743,12 +808,6 @@ gfx12_init_graphics_preamble_state(const struct ac_preamble_state *state,
    ac_pm4_set_reg(pm4, R_030A00_PA_SU_LINE_STIPPLE_VALUE, 0);
    ac_pm4_set_reg(pm4, R_030A04_PA_SC_LINE_STIPPLE_STATE, 0);
 
-   /* On GFX12, this seems to behave slightly differently. Programming the
-    * EXCLUSION fields to TRUE causes zero-area triangles to not pass the
-    * primitive clipping stage.
-    */
-   ac_pm4_set_reg(pm4, R_02882C_PA_SU_PRIM_FILTER_CNTL, 0);
-
    ac_pm4_set_reg(pm4, R_031128_SPI_GRP_LAUNCH_GUARANTEE_ENABLE,
                   S_031128_ENABLE(1) |
                   S_031128_GS_ASSIST_EN(1) |
