@@ -61,7 +61,7 @@
 #define RDECODE_CMDBUF_FLAGS_SCLR_COEF_BUFFER                        (0x00008000)
 #define RDECODE_CMDBUF_FLAGS_RECORD_TIMESTAMP                        (0x00010000)
 #define RDECODE_CMDBUF_FLAGS_REPORT_EVENT_STATUS                     (0x00020000)
-#define RDECODE_CMDBUF_FLAGS_RESERVED_SIZE_INFO_BUFFER               (0x00040000)
+#define RDECODE_CMDBUF_FLAGS_SUBSAMPLE_SIZE_INFO_BUFFER              (0x00040000)
 #define RDECODE_CMDBUF_FLAGS_LUMA_HIST_BUFFER                        (0x00080000)
 #define RDECODE_CMDBUF_FLAGS_SESSION_CONTEXT_BUFFER                  (0x00100000)
 #define RDECODE_CMDBUF_FLAGS_REF_BUFFER                              (0x00200000)
@@ -75,6 +75,7 @@
 #define RDECODE_CMD_BITSTREAM_BUFFER                        0x00000100
 #define RDECODE_CMD_IT_SCALING_TABLE_BUFFER                 0x00000204
 #define RDECODE_CMD_CONTEXT_BUFFER                          0x00000206
+#define RDECODE_CMD_SUBSAMPLE                               0x00000700
 #define RDECODE_CMD_WRITE_MEMORY                            0x00000800
 
 #define RDECODE_MSG_CREATE                                  0x00000000
@@ -146,6 +147,7 @@
 #define RDECODE_MESSAGE_VP9                                 0x0000000E
 #define RDECODE_MESSAGE_DYNAMIC_DPB                         0x00000010
 #define RDECODE_MESSAGE_AV1                                 0x00000011
+#define RDECODE_MESSAGE_DRM_KEYBLOB                         0x00000014
 
 #define RDECODE_FEEDBACK_PROFILING                          0x00000001
 
@@ -296,6 +298,7 @@
 #define TYPE7 7
 
 /* VP9 Frame header flags */
+#define RDECODE_FRAME_HDR_INFO_VP9_USE_FRAME_SIZE_AS_OFFSET_SHIFT     (15)
 #define RDECODE_FRAME_HDR_INFO_VP9_USE_UNCOMPRESSED_HEADER_SHIFT      (14)
 #define RDECODE_FRAME_HDR_INFO_VP9_USE_PREV_IN_FIND_MV_REFS_SHIFT     (13)
 #define RDECODE_FRAME_HDR_INFO_VP9_MODE_REF_DELTA_UPDATE_SHIFT        (12)
@@ -312,7 +315,7 @@
 #define RDECODE_FRAME_HDR_INFO_VP9_FRAME_TYPE_SHIFT                   (1)
 #define RDECODE_FRAME_HDR_INFO_VP9_SHOW_EXISTING_FRAME_SHIFT          (0)
 
-
+#define RDECODE_FRAME_HDR_INFO_VP9_USE_FRAME_SIZE_AS_OFFSET_MASK     (0x00008000)
 #define RDECODE_FRAME_HDR_INFO_VP9_USE_UNCOMPRESSED_HEADER_MASK      (0x00004000)
 #define RDECODE_FRAME_HDR_INFO_VP9_USE_PREV_IN_FIND_MV_REFS_MASK     (0x00002000)
 #define RDECODE_FRAME_HDR_INFO_VP9_MODE_REF_DELTA_UPDATE_MASK        (0x00001000)
@@ -353,6 +356,7 @@
 #define DRM_CMD_DRM_BYPASS_MASK        (0x80000000)
 
 /* Drm_cntl definitions */
+#define DRM_CNTL_CENC_ENABLE_SHIFT     (0)
 #define DRM_CNTL_ENC_BYTECNT_SHIFT     (6)
 #define DRM_CNTL_CLR_BYTECNT_SHIFT     (16)
 #define DRM_CNTL_BYPASS_SHIFT          (24)
@@ -469,8 +473,8 @@ typedef struct rvcn_decode_buffer_s {
    unsigned int it_sclr_table_buffer_address_lo;
    unsigned int sclr_target_buffer_address_hi;
    unsigned int sclr_target_buffer_address_lo;
-   unsigned int reserved_size_info_buffer_address_hi;
-   unsigned int reserved_size_info_buffer_address_lo;
+   unsigned int subsample_hi;
+   unsigned int subsample_lo;
    unsigned int mpeg2_pic_param_buffer_address_hi;
    unsigned int mpeg2_pic_param_buffer_address_lo;
    unsigned int mpeg2_mb_control_buffer_address_hi;
@@ -564,9 +568,17 @@ typedef struct rvcn_dec_message_drm_s {
    unsigned int	drm_offset;
    unsigned int	drm_cmd;
    unsigned int	drm_cntl;
-   unsigned int	drm_reserved;
+   unsigned int	drm_max_res;
+   unsigned int	drm_subsample_size;
 } rvcn_dec_message_drm_t;
 
+typedef struct rvcn_dec_message_drm_keyblob_s {
+   unsigned int contentKey[4];
+   unsigned int policyIndex;
+   unsigned int policyArray[32];
+   unsigned int signature[4];
+} rvcn_dec_message_drm_keyblob_t;
+
 typedef struct rvcn_dec_message_dynamic_dpb_s {
    unsigned int dpbConfigFlags;
    unsigned int dpbLumaPitch;
@@ -1074,7 +1086,6 @@ typedef struct rvcn_dec_message_av1_s {
    rvcn_dec_warped_motion_params_t global_motion[8];
    rvcn_dec_av1_tile_info_t tile_info[256];
    unsigned char reserved[3];
-   unsigned int av1_intrabc_workaround;
 } rvcn_dec_message_av1_t;
 
 typedef struct rvcn_dec_feature_index_s {
@@ -1250,6 +1261,9 @@ struct jpeg_params {
 #define RDECODE_VCN2_5_ENGINE_CNTL          0x9b4
 
 #define RDECODE_SESSION_CONTEXT_SIZE (128 * 1024)
+#define RDECODE_MAX_SUBSAMPLE_SIZE   (2048 * 2 * 4)
+
+void ac_vcn_vp9_fill_probs_table(void *ptr);
 
 unsigned ac_vcn_dec_calc_ctx_size_av1(unsigned av1_version);
 void ac_vcn_av1_init_probs(unsigned av1_version, uint8_t *prob);
