@@ -16,7 +16,8 @@
  */
 
 static void
-def_size(nir_def *def, unsigned *size, unsigned *align)
+def_size(nir_def *def, unsigned *size, unsigned *align,
+         nir_preamble_class *class)
 {
    unsigned bit_size = def->bit_size == 1 ? 32 : def->bit_size;
    /* Due to the implicit const file promotion we want to expand 16-bit values
@@ -25,6 +26,7 @@ def_size(nir_def *def, unsigned *size, unsigned *align)
     */
    *size = DIV_ROUND_UP(bit_size, 32) * def->num_components;
    *align = 1;
+   *class = nir_preamble_class_general;
 }
 
 static bool
@@ -319,7 +321,7 @@ ir3_nir_opt_preamble(nir_shader *nir, struct ir3_shader_variant *v)
       .subgroup_size_uniform = true,
       .load_workgroup_size_allowed = true,
       .def_size = def_size,
-      .preamble_storage_size = max_size,
+      .preamble_storage_size[nir_preamble_class_general] = max_size,
       .instr_cost_cb = instr_cost,
       .avoid_instr_cb = avoid_instr,
       .rewrite_cost_cb = rewrite_cost,
@@ -389,15 +391,18 @@ struct find_insert_block_state {
 };
 
 static bool
-find_dominated_block(nir_block *block, struct find_insert_block_state *state)
+find_dominated_src(nir_src *src, void *data)
 {
+   struct find_insert_block_state *state = data;
+   nir_block *src_block = src->ssa->parent_instr->block;
+
    if (!state->insert_block) {
-      state->insert_block = block;
+      state->insert_block = src_block;
       return true;
-   } else if (nir_block_dominates(state->insert_block, block)) {
-      state->insert_block = block;
+   } else if (nir_block_dominates(state->insert_block, src_block)) {
+      state->insert_block = src_block;
       return true;
-   } else if (nir_block_dominates(block, state->insert_block)) {
+   } else if (nir_block_dominates(src_block, state->insert_block)) {
       return true;
    } else {
       state->insert_block = NULL;
@@ -405,14 +410,6 @@ find_dominated_block(nir_block *block, struct find_insert_block_state *state)
    }
 }
 
-static bool
-find_dominated_src(nir_src *src, void *data)
-{
-   struct find_insert_block_state *state = data;
-   nir_block *src_block = src->ssa->parent_instr->block;
-   return find_dominated_block(src_block, state);
-}
-
 /* Find the block where instr can be inserted. This is the block that is
  * dominated by all its sources. If instr doesn't have any sources, return dflt.
  */
@@ -430,22 +427,6 @@ find_insert_block(nir_instr *instr, nir_block *dflt)
    return NULL;
 }
 
-static nir_block *
-find_insert_block_for_defs(nir_def *defs[], unsigned n)
-{
-   struct find_insert_block_state state = {
-      .insert_block = NULL,
-   };
-
-   for (unsigned i = 0; i < n; i++) {
-      if (!find_dominated_block(defs[i]->parent_instr->block, &state)) {
-         return NULL;
-      }
-   }
-
-   return state.insert_block;
-}
-
 static bool
 dominates(const nir_instr *old_instr, const nir_instr *new_instr)
 {
@@ -617,20 +598,6 @@ static bool
 emit_descriptor_prefetch(nir_builder *b, nir_instr *instr, nir_def **descs,
                          struct prefetch_state *state)
 {
-   nir_block *insert_block = descs[0]->parent_instr->block;
-
-   if (descs[1]) {
-      insert_block = find_insert_block_for_defs(descs, 2);
-
-      /* Since the preamble control flow was reconstructed from the original
-       * one, and the two descriptor defs were used by the same instruction, we
-       * must be able to find a legal place to insert the prefetch.
-       */
-      assert(insert_block);
-   }
-
-   b->cursor = nir_after_block(insert_block);
-
    if (instr->type == nir_instr_type_tex) {
       nir_tex_instr *tex = nir_instr_as_tex(instr);
       int sampler_index =
@@ -804,6 +771,8 @@ ir3_nir_opt_prefetch_descriptors(nir_shader *nir, struct ir3_shader_variant *v)
                                                   preamble_defs);
          }
 
+         /* ir3_rematerialize_def_for_preamble may have moved the cursor. */
+         b.cursor = nir_after_impl(preamble);
          progress |= emit_descriptor_prefetch(&b, instr, preamble_descs, &state);
 
          if (state.sampler.num_prefetches == MAX_PREFETCHES &&
@@ -862,11 +831,21 @@ ir3_nir_lower_preamble(nir_shader *nir, struct ir3_shader_variant *v)
          unsigned offset = preamble_base + nir_intrinsic_base(intrin);
          b->cursor = nir_before_instr(instr);
 
+         unsigned num_components = dest->num_components;
+
+         if (dest->bit_size == 64) {
+            num_components *= 2;
+         }
+
          nir_def *new_dest = nir_load_const_ir3(
-            b, dest->num_components, 32, nir_imm_int(b, 0), .base = offset);
+            b, num_components, 32, nir_imm_int(b, 0), .base = offset);
 
          if (dest->bit_size == 1) {
             new_dest = nir_i2b(b, new_dest);
+         } else if (dest->bit_size == 64) {
+            assert(num_components == 2);
+            new_dest = nir_pack_64_2x32_split(b, nir_channel(b, new_dest, 0),
+                                              nir_channel(b, new_dest, 1));
          } else if (dest->bit_size != 32) {
             if (all_uses_float(dest, true)) {
                assert(dest->bit_size == 16);
@@ -902,7 +881,9 @@ ir3_nir_lower_preamble(nir_shader *nir, struct ir3_shader_variant *v)
 
          if (src->bit_size == 1)
             src = nir_b2i32(b, src);
-         if (src->bit_size != 32) {
+         if (src->bit_size == 64) {
+            src = nir_unpack_64_2x32(b, src);
+         } else if (src->bit_size != 32) {
             if (BITSET_TEST(promoted_to_float, nir_intrinsic_base(intrin))){
                assert(src->bit_size == 16);
                src = nir_f2f32(b, src);
