@@ -15,8 +15,11 @@
 #include "util/perf/u_perfetto.h"
 #include "util/perf/u_perfetto_renderpass.h"
 
+#include "tu_cmd_buffer.h"
 #include "tu_tracepoints.h"
 #include "tu_tracepoints_perfetto.h"
+#include "vk_object.h"
+#include "vk_util.h"
 
 /* we can't include tu_knl.h and tu_device.h */
 
@@ -46,11 +49,13 @@ enum tu_stage_id {
    CMD_BUFFER_STAGE_ID,
    CMD_BUFFER_ANNOTATION_STAGE_ID,
    RENDER_PASS_STAGE_ID,
+   SECONDARY_CMD_BUFFER_STAGE_ID,
    CMD_BUFFER_ANNOTATION_RENDER_PASS_STAGE_ID,
    BINNING_STAGE_ID,
    GMEM_STAGE_ID,
    BYPASS_STAGE_ID,
    BLIT_STAGE_ID,
+   DRAW_STAGE_ID,
    COMPUTE_STAGE_ID,
    CLEAR_SYSMEM_STAGE_ID,
    CLEAR_GMEM_STAGE_ID,
@@ -75,11 +80,13 @@ static const struct {
    [CMD_BUFFER_STAGE_ID]     = { "Command Buffer" },
    [CMD_BUFFER_ANNOTATION_STAGE_ID]     = { "Annotation", "Command Buffer Annotation" },
    [RENDER_PASS_STAGE_ID]    = { "Render Pass" },
+   [SECONDARY_CMD_BUFFER_STAGE_ID] = { "Secondary Command Buffer" },
    [CMD_BUFFER_ANNOTATION_RENDER_PASS_STAGE_ID]    = { "Annotation", "Render Pass Command Buffer Annotation" },
    [BINNING_STAGE_ID]        = { "Binning", "Perform Visibility pass and determine target bins" },
    [GMEM_STAGE_ID]           = { "GMEM", "Rendering to GMEM" },
    [BYPASS_STAGE_ID]         = { "Bypass", "Rendering to system memory" },
    [BLIT_STAGE_ID]           = { "Blit", "Performing a Blit operation" },
+   [DRAW_STAGE_ID]           = { "Draw", "Performing a graphics-pipeline draw" },
    [COMPUTE_STAGE_ID]        = { "Compute", "Compute job" },
    [CLEAR_SYSMEM_STAGE_ID]   = { "Clear Sysmem", "" },
    [CLEAR_GMEM_STAGE_ID]     = { "Clear GMEM", "Per-tile (GMEM) clear" },
@@ -107,12 +114,8 @@ static uint64_t last_suspend_count;
 static uint64_t gpu_max_timestamp;
 static uint64_t gpu_timestamp_offset;
 
-struct TuRenderpassIncrementalState {
-   bool was_cleared = true;
-};
-
 struct TuRenderpassTraits : public perfetto::DefaultDataSourceTraits {
-   using IncrementalStateType = TuRenderpassIncrementalState;
+   using IncrementalStateType = MesaRenderpassIncrementalState;
 };
 
 class TuRenderpassDataSource : public MesaRenderpassDataSource<TuRenderpassDataSource,
@@ -138,8 +141,14 @@ PERFETTO_DECLARE_DATA_SOURCE_STATIC_MEMBERS(TuRenderpassDataSource);
 PERFETTO_DEFINE_DATA_SOURCE_STATIC_MEMBERS(TuRenderpassDataSource);
 
 static void
-send_descriptors(TuRenderpassDataSource::TraceContext &ctx)
+setup_incremental_state(TuRenderpassDataSource::TraceContext &ctx)
 {
+   auto state = ctx.GetIncrementalState();
+   if (!state->was_cleared)
+      return;
+
+   state->was_cleared = false;
+
    PERFETTO_LOG("Sending renderstage descriptors");
 
    auto packet = ctx.NewTracePacket();
@@ -257,6 +266,11 @@ stage_end(struct tu_device *dev, uint64_t ts_ns, enum tu_stage_id stage_id,
    if (!stage)
       return;
 
+   uint64_t duration = ts_ns - stage->start_ts;
+   /* Zero duration can only happen when tracepoints did not happen on GPU. */
+   if (duration == 0)
+      return;
+
    if (stage->stage_id != stage_id) {
       PERFETTO_ELOG("stage %d ended while stage %d is expected",
             stage_id, stage->stage_id);
@@ -271,10 +285,7 @@ stage_end(struct tu_device *dev, uint64_t ts_ns, enum tu_stage_id stage_id,
       return;
 
    TuRenderpassDataSource::Trace([=](TuRenderpassDataSource::TraceContext tctx) {
-      if (auto state = tctx.GetIncrementalState(); state->was_cleared) {
-         send_descriptors(tctx);
-         state->was_cleared = false;
-      }
+      setup_incremental_state(tctx);
 
       auto packet = tctx.NewTracePacket();
 
@@ -334,6 +345,8 @@ extern "C" {
 void
 tu_perfetto_init(void)
 {
+   util_perfetto_init();
+
    {
    perfetto::DataSourceDescriptor dsd;
 #if DETECT_OS_ANDROID
@@ -509,11 +522,13 @@ tu_perfetto_end_submit(struct tu_queue *queue,
    }
 
 CREATE_EVENT_CALLBACK(cmd_buffer, CMD_BUFFER_STAGE_ID)
+CREATE_EVENT_CALLBACK(secondary_cmd_buffer, SECONDARY_CMD_BUFFER_STAGE_ID)
 CREATE_EVENT_CALLBACK(render_pass, RENDER_PASS_STAGE_ID)
 CREATE_EVENT_CALLBACK(binning_ib, BINNING_STAGE_ID)
 CREATE_EVENT_CALLBACK(draw_ib_gmem, GMEM_STAGE_ID)
 CREATE_EVENT_CALLBACK(draw_ib_sysmem, BYPASS_STAGE_ID)
 CREATE_EVENT_CALLBACK(blit, BLIT_STAGE_ID)
+CREATE_EVENT_CALLBACK(draw, DRAW_STAGE_ID)
 CREATE_EVENT_CALLBACK(compute, COMPUTE_STAGE_ID)
 CREATE_EVENT_CALLBACK(compute_indirect, COMPUTE_STAGE_ID)
 CREATE_EVENT_CALLBACK(generic_clear, GENERIC_CLEAR_STAGE_ID)
@@ -658,6 +673,28 @@ tu_perfetto_log_destroy_image(struct tu_device *dev, struct tu_image *image)
 
 
 
+void
+tu_perfetto_set_debug_utils_object_name(const VkDebugUtilsObjectNameInfoEXT *pNameInfo)
+{
+   TuRenderpassDataSource::Trace([=](auto tctx) {
+      /* Do we need this for SEQ_INCREMENTAL_STATE_CLEARED for the object name to stick? */
+      setup_incremental_state(tctx);
+
+      tctx.GetDataSourceLocked()->SetDebugUtilsObjectNameEXT(tctx, pNameInfo);
+   });
+}
+
+void
+tu_perfetto_refresh_debug_utils_object_name(const struct vk_object_base *object)
+{
+   TuRenderpassDataSource::Trace([=](auto tctx) {
+      /* Do we need this for SEQ_INCREMENTAL_STATE_CLEARED for the object name to stick? */
+      setup_incremental_state(tctx);
+
+      tctx.GetDataSourceLocked()->RefreshSetDebugUtilsObjectNameEXT(tctx, object);
+   });
+}
+
 #ifdef __cplusplus
 }
 #endif
