@@ -105,22 +105,20 @@ trace_screen_get_device_vendor(struct pipe_screen *_screen)
 }
 
 
-static const void *
+static const struct nir_shader_compiler_options *
 trace_screen_get_compiler_options(struct pipe_screen *_screen,
-                                  enum pipe_shader_ir ir,
                                   enum pipe_shader_type shader)
 {
    struct trace_screen *tr_scr = trace_screen(_screen);
    struct pipe_screen *screen = tr_scr->screen;
-   const void *result;
+   const struct nir_shader_compiler_options *result;
 
    trace_dump_call_begin("pipe_screen", "get_compiler_options");
 
    trace_dump_arg(ptr, screen);
-   trace_dump_arg_enum(pipe_shader_ir, ir);
    trace_dump_arg_enum(pipe_shader_type, shader);
 
-   result = screen->get_compiler_options(screen, ir, shader);
+   result = screen->get_compiler_options(screen, shader);
 
    trace_dump_ret(ptr, result);
 
@@ -855,6 +853,24 @@ trace_screen_resource_get_info(struct pipe_screen *_screen,
    trace_dump_call_end();
 }
 
+static uint64_t
+trace_screen_resource_get_address(struct pipe_screen *_screen,
+                                  struct pipe_resource *resource)
+{
+   struct trace_screen *tr_screen = trace_screen(_screen);
+   struct pipe_screen *screen = tr_screen->screen;
+
+   trace_dump_call_begin("pipe_screen", "resource_get_address");
+   trace_dump_arg(ptr, screen);
+   trace_dump_arg(ptr, resource);
+
+   uint64_t res = screen->resource_get_address(screen, resource);
+
+   trace_dump_ret(uint, res);
+   trace_dump_call_end();
+   return res;
+}
+
 static struct pipe_resource *
 trace_screen_resource_from_memobj(struct pipe_screen *_screen,
                                   const struct pipe_resource *templ,
@@ -949,12 +965,15 @@ trace_screen_fence_get_fd(struct pipe_screen *_screen,
 {
    struct trace_screen *tr_scr = trace_screen(_screen);
    struct pipe_screen *screen = tr_scr->screen;
-   int result = screen->fence_get_fd(screen, fence);
+   int result;
 
    trace_dump_call_begin("pipe_screen", "fence_get_fd");
 
    trace_dump_arg(ptr, screen);
    trace_dump_arg(ptr, fence);
+
+   result = screen->fence_get_fd(screen, fence);
+
    trace_dump_ret(int, result);
 
    trace_dump_call_end();
@@ -1311,22 +1330,6 @@ static void trace_screen_vertex_state_destroy(struct pipe_screen *_screen,
    screen->vertex_state_destroy(screen, state);
 }
 
-static void trace_screen_set_fence_timeline_value(struct pipe_screen *_screen,
-                                                  struct pipe_fence_handle *fence,
-                                                  uint64_t value)
-{
-   struct trace_screen *tr_scr = trace_screen(_screen);
-   struct pipe_screen *screen = tr_scr->screen;
-
-   trace_dump_call_begin("pipe_screen", "set_fence_timeline_value");
-   trace_dump_arg(ptr, screen);
-   trace_dump_arg(ptr, fence);
-   trace_dump_arg(uint, value);
-   trace_dump_call_end();
-
-   screen->set_fence_timeline_value(screen, fence, value);
-}
-
 static void trace_screen_query_compression_rates(struct pipe_screen *_screen,
                                                  enum pipe_format format,
                                                  int max, uint32_t *rates,
@@ -1381,6 +1384,61 @@ static void trace_screen_query_compression_modifiers(struct pipe_screen *_screen
    trace_dump_call_end();
 }
 
+static struct pipe_vm_allocation *
+trace_screen_alloc_vm(struct pipe_screen *_screen, uint64_t start,
+                      uint64_t size)
+{
+   struct trace_screen *tr_scr = trace_screen(_screen);
+   struct pipe_screen *screen = tr_scr->screen;
+
+   trace_dump_call_begin("pipe_screen", "alloc_vm");
+   trace_dump_arg(ptr, screen);
+   trace_dump_arg(uint, start);
+   trace_dump_arg(uint, size);
+
+   struct pipe_vm_allocation *res = screen->alloc_vm(screen, start, size);
+
+   trace_dump_ret(vm_allocation, res);
+   trace_dump_call_end();
+
+   return res;
+}
+
+static void
+trace_screen_free_vm(struct pipe_screen *_screen, struct pipe_vm_allocation *alloc)
+{
+   struct trace_screen *tr_scr = trace_screen(_screen);
+   struct pipe_screen *screen = tr_scr->screen;
+
+   trace_dump_call_begin("pipe_screen", "free_vm");
+   trace_dump_arg(ptr, screen);
+   trace_dump_arg(vm_allocation, alloc);
+   trace_dump_call_end();
+
+   screen->free_vm(screen, alloc);
+}
+
+static bool
+trace_screen_resource_assign_vma(struct pipe_screen *_screen,
+                                 struct pipe_resource *resource,
+                                 uint64_t address)
+{
+   struct trace_screen *tr_scr = trace_screen(_screen);
+   struct pipe_screen *screen = tr_scr->screen;
+
+   trace_dump_call_begin("pipe_screen", "resource_assign_vma");
+   trace_dump_arg(ptr, screen);
+   trace_dump_arg(ptr, resource);
+   trace_dump_arg(uint, address);
+
+   bool res = screen->resource_assign_vma(screen, resource, address);
+
+   trace_dump_ret(bool, res);
+   trace_dump_call_end();
+
+   return res;
+}
+
 bool
 trace_enabled(void)
 {
@@ -1472,6 +1530,7 @@ trace_screen_create(struct pipe_screen *screen)
    tr_scr->base.resource_get_handle = trace_screen_resource_get_handle;
    SCR_INIT(resource_get_param);
    SCR_INIT(resource_get_info);
+   SCR_INIT(resource_get_address);
    SCR_INIT(resource_from_memobj);
    SCR_INIT(resource_changed);
    tr_scr->base.resource_destroy = trace_screen_resource_destroy;
@@ -1492,10 +1551,12 @@ trace_screen_create(struct pipe_screen *screen)
    SCR_INIT(vertex_state_destroy);
    tr_scr->base.transfer_helper = screen->transfer_helper;
    SCR_INIT(get_sparse_texture_virtual_page_size);
-   SCR_INIT(set_fence_timeline_value);
    SCR_INIT(driver_thread_add_job);
    SCR_INIT(query_compression_rates);
    SCR_INIT(query_compression_modifiers);
+   SCR_INIT(alloc_vm);
+   SCR_INIT(free_vm);
+   SCR_INIT(resource_assign_vma);
    tr_scr->base.get_driver_pipe_screen = tr_get_driver_pipe_screen;
 
    tr_scr->screen = screen;
