@@ -30,11 +30,8 @@
 #include "util/u_inlines.h"
 #include "util/u_memory.h"
 #include "util/u_string.h"
-#include "util/u_dl.h"
-#include "util/u_file.h"
 #include "util/xmlconfig.h"
 #include "util/driconf.h"
-#include "util/perf/cpu_trace.h"
 
 #include <string.h>
 
@@ -43,8 +40,6 @@
 #define PATH_MAX _MAX_PATH
 #endif
 
-#define MODULE_PREFIX "pipe_"
-
 static int (*backends[])(struct pipe_loader_device **, int) = {
 #ifdef HAVE_LIBDRM
    &pipe_loader_drm_probe,
@@ -174,8 +169,6 @@ pipe_loader_create_screen_vk(struct pipe_loader_device *dev, bool sw_vk, bool dr
 {
    struct pipe_screen_config config;
 
-   util_cpu_trace_init();
-
    pipe_loader_load_options(dev);
    config.driver_name_is_inferred = driver_name_is_inferred;
    config.options_info = &dev->option_info;
@@ -189,37 +182,3 @@ pipe_loader_create_screen(struct pipe_loader_device *dev, bool driver_name_is_in
 {
    return pipe_loader_create_screen_vk(dev, false, driver_name_is_inferred);
 }
-
-struct util_dl_library *
-pipe_loader_find_module(const char *driver_name,
-                        const char *library_paths)
-{
-   struct util_dl_library *lib;
-   const char *next;
-   char path[PATH_MAX];
-   int len, ret;
-
-   for (next = library_paths; *next; library_paths = next + 1) {
-      next = strchrnul(library_paths, ':');
-      len = next - library_paths;
-
-      if (len)
-         ret = snprintf(path, sizeof(path), "%.*s/%s%s%s",
-                        len, library_paths,
-                        MODULE_PREFIX, driver_name, UTIL_DL_EXT);
-      else
-         ret = snprintf(path, sizeof(path), "%s%s%s",
-                        MODULE_PREFIX, driver_name, UTIL_DL_EXT);
-
-      if (ret > 0 && ret < sizeof(path) && u_file_access(path, 0) != -1) {
-         lib = util_dl_open(path);
-         if (lib) {
-            return lib;
-         }
-         fprintf(stderr, "ERROR: Failed to load pipe driver at `%s': %s\n",
-                         path, util_dl_error());
-      }
-   }
-
-   return NULL;
-}
