@@ -22,12 +22,8 @@ static uint64_t next_clock_sync_ns; /* cpu time of next clk sync */
  */
 static uint64_t sync_gpu_ts;
 
-struct FdRenderpassIncrementalState {
-   bool was_cleared = true;
-};
-
 struct FdRenderpassTraits : public perfetto::DefaultDataSourceTraits {
-   using IncrementalStateType = FdRenderpassIncrementalState;
+   using IncrementalStateType = MesaRenderpassIncrementalState;
 };
 
 class FdRenderpassDataSource : public MesaRenderpassDataSource<FdRenderpassDataSource, FdRenderpassTraits> {
@@ -265,6 +261,8 @@ extern "C" {
 void
 fd_perfetto_init(void)
 {
+   util_perfetto_init();
+
    perfetto::DataSourceDescriptor dsd;
 #if DETECT_OS_ANDROID
    // Android tooling expects this data source name
