@@ -26,7 +26,7 @@
  **************************************************************************/
 
 
-#include "pipe/p_config.h"
+#include "util/detect.h"
 
 #include "util/u_math.h"
 #include "util/u_cpu_detect.h"
@@ -37,13 +37,14 @@
 #include "lp_debug.h"
 
 
-#if defined(PIPE_ARCH_SSE)
+#if DETECT_ARCH_SSE
 
 #include <emmintrin.h>
 
 
 static void
 no_op(const struct lp_jit_context *context,
+      const struct lp_jit_resources *resources,
       uint32_t x,
       uint32_t y,
       uint32_t facing,
@@ -142,6 +143,7 @@ opaque_color(uint8_t **cbufs, unsigned *strides,
  */
 static void
 red(const struct lp_jit_context *context,
+    const struct lp_jit_resources *resources,
     uint32_t x,
     uint32_t y,
     uint32_t facing,
@@ -169,6 +171,7 @@ red(const struct lp_jit_context *context,
  */
 static void
 green(const struct lp_jit_context *context,
+      const struct lp_jit_resources *resources,
       uint32_t x,
       uint32_t y,
       uint32_t facing,
