@@ -46,13 +46,14 @@
 #include "drm-uapi/panfrost_drm.h"
 
 #include "decode.h"
+#include "pan_afbc.h"
+#include "pan_afrc.h"
 #include "pan_bo.h"
 #include "pan_fence.h"
 #include "pan_public.h"
 #include "pan_resource.h"
 #include "pan_screen.h"
 #include "pan_shader.h"
-#include "pan_texture.h"
 #include "pan_util.h"
 
 #include "pan_context.h"
@@ -61,24 +62,24 @@
 
 /* clang-format off */
 static const struct debug_named_value panfrost_debug_options[] = {
-   {"perf",       PAN_DBG_PERF,     "Enable performance warnings"},
-   {"trace",      PAN_DBG_TRACE,    "Trace the command stream"},
-   {"dirty",      PAN_DBG_DIRTY,    "Always re-emit all state"},
-   {"sync",       PAN_DBG_SYNC,     "Wait for each job's completion and abort on GPU faults"},
-   {"nofp16",     PAN_DBG_NOFP16,    "Disable 16-bit support"},
-   {"gl3",        PAN_DBG_GL3,      "Enable experimental GL 3.x implementation, up to 3.3"},
-   {"noafbc",     PAN_DBG_NO_AFBC,  "Disable AFBC support"},
-   {"nocrc",      PAN_DBG_NO_CRC,   "Disable transaction elimination"},
-   {"msaa16",     PAN_DBG_MSAA16,   "Enable MSAA 8x and 16x support"},
-   {"linear",     PAN_DBG_LINEAR,   "Force linear textures"},
-   {"nocache",    PAN_DBG_NO_CACHE, "Disable BO cache"},
-   {"dump",       PAN_DBG_DUMP,     "Dump all graphics memory"},
+   {"perf",       PAN_DBG_PERF,       "Enable performance warnings"},
+   {"trace",      PAN_DBG_TRACE,      "Trace the command stream"},
+   {"dirty",      PAN_DBG_DIRTY,      "Always re-emit all state"},
+   {"sync",       PAN_DBG_SYNC,       "Wait for each job's completion and abort on GPU faults"},
+   {"nofp16",     PAN_DBG_NOFP16,     "Disable 16-bit support"},
+   {"gl3",        PAN_DBG_GL3,        "Enable experimental GL 3.x implementation, up to 3.3"},
+   {"noafbc",     PAN_DBG_NO_AFBC,    "Disable AFBC support"},
+   {"nocrc",      PAN_DBG_NO_CRC,     "Disable transaction elimination"},
+   {"linear",     PAN_DBG_LINEAR,     "Force linear textures"},
+   {"strict_import", PAN_DBG_STRICT_IMPORT, "Use the explicit WSI stride and fail if it's not properly aligned"},
+   {"nocache",    PAN_DBG_NO_CACHE,   "Disable BO cache"},
+   {"dump",       PAN_DBG_DUMP,       "Dump all graphics memory"},
 #ifdef PAN_DBG_OVERFLOW
-   {"overflow",   PAN_DBG_OVERFLOW, "Check for buffer overflows in pool uploads"},
+   {"overflow",   PAN_DBG_OVERFLOW,   "Check for buffer overflows in pool uploads"},
 #endif
-   {"yuv",        PAN_DBG_YUV,      "Tint YUV textures with blue for 1-plane and green for 2-plane"},
-   {"forcepack",  PAN_DBG_FORCE_PACK,  "Force packing of AFBC textures on upload"},
-   {"cs",         PAN_DBG_CS,       "Enable extra checks in command stream"},
+   {"yuv",        PAN_DBG_YUV,        "Tint YUV textures with blue for 1-plane and green for 2-plane"},
+   {"forcepack",  PAN_DBG_FORCE_PACK, "Pack AFBC textures progressively in the background"},
+   {"cs",         PAN_DBG_CS,         "Enable extra checks in command stream"},
    DEBUG_NAMED_VALUE_END
 };
 /* clang-format on */
@@ -125,13 +126,42 @@ from_kmod_group_allow_priority_flags(
 static uint32_t
 pipe_to_pan_bind_flags(uint32_t pipe_bind_flags)
 {
-   static_assert(PIPE_BIND_DEPTH_STENCIL == PAN_BIND_DEPTH_STENCIL, "");
-   static_assert(PIPE_BIND_RENDER_TARGET == PAN_BIND_RENDER_TARGET, "");
-   static_assert(PIPE_BIND_SAMPLER_VIEW == PAN_BIND_SAMPLER_VIEW, "");
-   static_assert(PIPE_BIND_VERTEX_BUFFER == PAN_BIND_VERTEX_BUFFER, "");
+   uint32_t pan_bind_flags = 0;
 
-   return pipe_bind_flags & (PAN_BIND_DEPTH_STENCIL | PAN_BIND_RENDER_TARGET |
-                             PAN_BIND_VERTEX_BUFFER | PAN_BIND_SAMPLER_VIEW);
+   if (pipe_bind_flags & PIPE_BIND_DEPTH_STENCIL)
+      pan_bind_flags |= PAN_BIND_DEPTH_STENCIL;
+   if (pipe_bind_flags & PIPE_BIND_RENDER_TARGET)
+      pan_bind_flags |= PAN_BIND_RENDER_TARGET;
+   if (pipe_bind_flags & PIPE_BIND_VERTEX_BUFFER)
+      pan_bind_flags |= PAN_BIND_VERTEX_BUFFER;
+   if (pipe_bind_flags & PIPE_BIND_SAMPLER_VIEW)
+      pan_bind_flags |= PAN_BIND_SAMPLER_VIEW;
+
+   return pan_bind_flags;
+}
+
+static unsigned
+get_max_msaa(struct panfrost_device *dev, enum pipe_format format)
+{
+   unsigned max_tib_size = pan_get_max_tib_size(dev->arch, dev->model);
+   unsigned max_cbuf_atts = pan_get_max_cbufs(dev->arch, max_tib_size);
+   unsigned format_size = util_format_get_blocksize(format);
+
+   unsigned max_msaa = pan_get_max_msaa(dev->arch, max_tib_size,
+                                        max_cbuf_atts, format_size);
+   assert(format_size > 16 || max_msaa >= 4);
+
+   /* t760 (GPU ID 0x750 - not a typo) has a HW issue in versions before
+    * the r1p0 version, which prevents 16x MSAA from working properly.
+    */
+   if (panfrost_device_gpu_prod_id(dev) == 0x750 &&
+       panfrost_device_gpu_rev(dev) < 0x1000)
+      max_msaa = MIN2(max_msaa, 8);
+
+   if (dev->model->quirks.max_4x_msaa)
+      max_msaa = MIN2(max_msaa, 4);
+
+   return max_msaa;
 }
 
 /**
@@ -148,23 +178,13 @@ panfrost_is_format_supported(struct pipe_screen *screen,
 {
    struct panfrost_device *dev = pan_device(screen);
 
-   /* MSAA 2x gets rounded up to 4x. MSAA 8x/16x only supported on v5+.
-    * TODO: debug MSAA 8x/16x */
+   unsigned max_msaa = get_max_msaa(dev, format);
+   if (!util_is_power_of_two_or_zero(sample_count) ||
+       MAX2(sample_count, 1) > max_msaa)
+      return false;
 
-   switch (sample_count) {
-   case 0:
-   case 1:
-   case 4:
-      break;
-   case 8:
-   case 16:
-      if (dev->debug & PAN_DBG_MSAA16)
-         break;
-      else
-         return false;
-   default:
+   if (sample_count == 2 && dev->arch < 12)
       return false;
-   }
 
    if (MAX2(sample_count, 1) != MAX2(storage_sample_count, 1))
       return false;
@@ -176,7 +196,7 @@ panfrost_is_format_supported(struct pipe_screen *screen,
    /* Check we support the format with the given bind */
 
    unsigned pan_bind_flags = pipe_to_pan_bind_flags(bind);
-   struct panfrost_format fmt = dev->formats[format];
+   struct pan_format fmt = dev->formats[format];
    unsigned fmt_bind_flags = fmt.bind;
 
    /* Also check that compressed texture formats are supported on this
@@ -222,7 +242,69 @@ panfrost_query_compression_rates(struct pipe_screen *screen,
       return;
    }
 
-   *count = panfrost_afrc_query_rates(format, max, rates);
+   *count = pan_afrc_query_rates(format, max, rates);
+}
+
+struct panfrost_yuv_format_lowering {
+   unsigned nres;
+   enum pipe_format res_formats[3];
+};
+
+static struct panfrost_yuv_format_lowering
+panfrost_lower_yuv_format(struct panfrost_device *dev,
+                          enum pipe_format format)
+{
+   assert(util_format_is_yuv(format));
+
+   switch (format) {
+#define SINGLE_RES(__in, __out)                                                \
+   case PIPE_FORMAT_##__in:                                                    \
+      if (dev->formats[PIPE_FORMAT_##__out].bind & PAN_BIND_SAMPLER_VIEW) {    \
+         return (struct panfrost_yuv_format_lowering){                         \
+            .nres = 1,                                                         \
+            .res_formats[0] = PIPE_FORMAT_##__out,                             \
+         };                                                                    \
+      }                                                                        \
+      break;
+
+   SINGLE_RES(AYUV, RGBA8888_UNORM)
+   SINGLE_RES(XYUV, RGBX8888_UNORM)
+   SINGLE_RES(YUYV, R8G8_R8B8_UNORM)
+   SINGLE_RES(UYVY, G8R8_B8R8_UNORM)
+   SINGLE_RES(YVYU, R8B8_R8G8_UNORM)
+   SINGLE_RES(VYUY, B8R8_G8R8_UNORM)
+   SINGLE_RES(NV12, R8_G8B8_420_UNORM)
+   SINGLE_RES(NV21, R8_B8G8_420_UNORM)
+   SINGLE_RES(NV16, R8_G8B8_422_UNORM)
+   SINGLE_RES(NV15, R10_G10B10_420_UNORM)
+   SINGLE_RES(NV20, R10_G10B10_422_UNORM)
+   SINGLE_RES(IYUV, R8_G8_B8_420_UNORM)
+   SINGLE_RES(YV12, R8_B8_G8_420_UNORM)
+   SINGLE_RES(Y8U8V8_420_UNORM_PACKED, R8G8B8_420_UNORM_PACKED)
+   SINGLE_RES(Y10U10V10_420_UNORM_PACKED, R10G10B10_420_UNORM_PACKED)
+
+#undef SINGLE_RES
+
+   default:
+      break;
+   }
+
+   struct panfrost_yuv_format_lowering lowering = {0};
+   unsigned nplanes =  util_format_get_num_planes(format);
+   for (unsigned i = 0; i < nplanes; i++) {
+      lowering.res_formats[lowering.nres++] =
+         util_format_get_plane_format(format, i);
+
+      /* If there's no YUV-as-RGB lowering available, the original YUV format
+       * will be returned, and only LINEAR will be allowed. */
+      if (i == 0 && lowering.res_formats[i] == format)
+         return lowering;
+
+      /* If plane0 got lowered, so should planeX. */
+      assert(lowering.res_formats[i] != format);
+   }
+
+   return lowering;
 }
 
 /* We always support linear and tiled operations, both external and internal.
@@ -235,48 +317,116 @@ panfrost_walk_dmabuf_modifiers(struct pipe_screen *screen,
                                uint64_t *modifiers, unsigned int *external_only,
                                int *out_count, uint64_t test_modifier, bool allow_afrc)
 {
-   /* Query AFBC status */
    struct panfrost_device *dev = pan_device(screen);
-   bool afbc =
-      dev->has_afbc && panfrost_format_supports_afbc(dev->arch, format);
-   bool ytr = panfrost_afbc_can_ytr(format);
-   bool tiled_afbc = panfrost_afbc_can_tile(dev->arch);
-   bool afrc = allow_afrc && dev->has_afrc && panfrost_format_supports_afrc(format);
+   bool is_yuv = util_format_is_yuv(format);
+   struct panfrost_yuv_format_lowering yuv_lowering = {0};
 
+   if (is_yuv) {
+      yuv_lowering =
+         panfrost_lower_yuv_format(dev, format);
+
+      if (yuv_lowering.nres == 1)
+         format = yuv_lowering.res_formats[0];
+   }
+
+   /* Query AFBC status */
+   bool afbc = dev->has_afbc;
+   bool ytr = afbc && !is_yuv;
+   bool tiled_afbc = pan_afbc_can_tile(dev->arch);
+   bool afrc = allow_afrc && dev->has_afrc;
+
+   if (is_yuv && yuv_lowering.nres > 1) {
+      for (unsigned i = 0; i < yuv_lowering.nres; i++) {
+         enum pipe_format plane_format = yuv_lowering.res_formats[i];
+
+         afbc &= pan_afbc_supports_format(dev->arch, plane_format);
+      }
+   } else {
+      afbc &= pan_afbc_supports_format(dev->arch, format);
+      ytr &= pan_afbc_can_ytr(format);
+      afrc &= !is_yuv && pan_afrc_supports_format(format);
+   }
+
+   PANFROST_EMULATED_MODIFIERS(emulated_mods);
+   PAN_SUPPORTED_MODIFIERS(native_mods);
    unsigned count = 0;
 
-   for (unsigned i = 0; i < PAN_MODIFIER_COUNT; ++i) {
-      if (drm_is_afbc(pan_best_modifiers[i])) {
+   for (unsigned i = 0; i < ARRAY_SIZE(native_mods); ++i) {
+      if (drm_is_afbc(native_mods[i])) {
          if (!afbc)
             continue;
 
-         if ((pan_best_modifiers[i] & AFBC_FORMAT_MOD_SPLIT) &&
-             !panfrost_afbc_can_split(dev->arch, format, pan_best_modifiers[i]))
+         if ((native_mods[i] & AFBC_FORMAT_MOD_SPLIT)) {
+            unsigned nplanes = util_format_get_num_planes(format);
+            bool can_split = true;
+
+            for (unsigned p = 0; p < nplanes; p++) {
+               if (is_yuv && yuv_lowering.nres > 1) {
+                  can_split &= pan_afbc_can_split(
+                     dev->arch, yuv_lowering.res_formats[p], native_mods[i], 0);
+               } else {
+                  can_split &=
+                     pan_afbc_can_split(dev->arch, format, native_mods[i], p);
+               }
+            }
+
+            if (!can_split)
+               continue;
+         }
+
+         if ((native_mods[i] & AFBC_FORMAT_MOD_YTR) && !ytr)
             continue;
 
-         if ((pan_best_modifiers[i] & AFBC_FORMAT_MOD_YTR) && !ytr)
-            continue;
-
-         if ((pan_best_modifiers[i] & AFBC_FORMAT_MOD_TILED) && !tiled_afbc)
+         if ((native_mods[i] & AFBC_FORMAT_MOD_TILED) && !tiled_afbc)
             continue;
       }
 
-      if (drm_is_afrc(pan_best_modifiers[i]) && !afrc)
+      if (drm_is_afrc(native_mods[i]) && !afrc)
          continue;
 
-      if (drm_is_mtk_tiled(format, pan_best_modifiers[i]) &&
+      if (drm_is_mtk_tiled(native_mods[i]) &&
+          !panfrost_format_supports_mtk_tiled(format))
+         continue;
+
+      /* If the format is still YUV after lowering, the SW emulation might
+       * involve plane aliasing which we can't do with U_TILED. */
+      if (util_format_is_yuv(format) &&
+          native_mods[i] == DRM_FORMAT_MOD_ARM_16X16_BLOCK_U_INTERLEAVED)
+         continue;
+
+      /* Some formats only work with AFBC. */
+      if ((native_mods[i] == DRM_FORMAT_MOD_LINEAR ||
+           native_mods[i] == DRM_FORMAT_MOD_ARM_16X16_BLOCK_U_INTERLEAVED) &&
+          !pan_u_tiled_or_linear_supports_format(format))
+         continue;
+
+      if (test_modifier != DRM_FORMAT_MOD_INVALID &&
+          test_modifier != native_mods[i])
+         continue;
+
+      if (max > (int)count) {
+         modifiers[count] = native_mods[i];
+
+         if (external_only)
+            external_only[count] = is_yuv;
+      }
+      count++;
+   }
+
+   for (unsigned i = 0; i < ARRAY_SIZE(emulated_mods); ++i) {
+      if (drm_is_mtk_tiled(emulated_mods[i]) &&
           !panfrost_format_supports_mtk_tiled(format))
          continue;
 
       if (test_modifier != DRM_FORMAT_MOD_INVALID &&
-          test_modifier != pan_best_modifiers[i])
+          test_modifier != emulated_mods[i])
          continue;
 
       if (max > (int)count) {
-         modifiers[count] = pan_best_modifiers[i];
+         modifiers[count] = emulated_mods[i];
 
          if (external_only)
-            external_only[count] = drm_is_mtk_tiled(format, modifiers[count]);
+            external_only[count] = true;
       }
       count++;
    }
@@ -309,7 +459,7 @@ panfrost_query_compression_modifiers(struct pipe_screen *screen,
                                      DRM_FORMAT_MOD_INVALID,
                                      false /* disallow afrc */);
    else if (dev->has_afrc)
-      *count = panfrost_afrc_get_modifiers(format, rate, max, modifiers);
+      *count = pan_afrc_get_modifiers(format, rate, max, modifiers);
    else
       *count = 0;  /* compression requested but not supported */
 }
@@ -364,7 +514,7 @@ panfrost_init_shader_caps(struct panfrost_screen *screen)
       caps->max_tex_indirections = 16384; /* arbitrary */
       caps->max_control_flow_depth = 1024; /* arbitrary */
       /* Used as ABI on Midgard */
-      caps->max_inputs = 16;
+      caps->max_inputs = dev->arch >= 9 ? 32 : 16;
       caps->max_outputs = i == PIPE_SHADER_FRAGMENT ? 8 : PIPE_MAX_ATTRIBS;
       caps->max_temps = 256; /* arbitrary */
       caps->max_const_buffer0_size = 16 * 1024 * sizeof(float);
@@ -380,6 +530,7 @@ panfrost_init_shader_caps(struct panfrost_screen *screen)
       caps->glsl_16bit_consts = !is_nofp16;
       caps->fp16_derivatives =
       caps->fp16_const_buffers = dev->arch >= 6 && !is_nofp16;
+      caps->glsl_16bit_load_dst = true;
       /* Blocked on https://gitlab.freedesktop.org/mesa/mesa/-/issues/6075 */
       caps->int16 = false;
       STATIC_ASSERT(PIPE_MAX_SAMPLERS < 0x10000);
@@ -446,20 +597,17 @@ panfrost_init_compute_caps(struct panfrost_screen *screen)
     * things so it matches kmod VA range limitations.
     */
    uint64_t user_va_start =
-      panfrost_clamp_to_usable_va_range(dev->kmod.dev, PAN_VA_USER_START);
+      pan_clamp_to_usable_va_range(dev->kmod.dev, PAN_VA_USER_START);
    uint64_t user_va_end =
-      panfrost_clamp_to_usable_va_range(dev->kmod.dev, PAN_VA_USER_END);
+      pan_clamp_to_usable_va_range(dev->kmod.dev, PAN_VA_USER_END);
 
    /* We cannot support more than the VA limit */
    caps->max_global_size =
    caps->max_mem_alloc_size = MIN2(available_ram, user_va_end - user_va_start);
 
    caps->max_local_size = 32768;
-   caps->max_private_size =
-   caps->max_input_size = 4096;
    caps->max_clock_frequency = 800; /* MHz -- TODO */
    caps->max_compute_units = dev->core_count;
-   caps->images_supported = true;
    caps->subgroup_sizes = pan_subgroup_size(dev->arch);
    caps->max_variable_threads_per_block = 1024; // TODO
 }
@@ -476,8 +624,8 @@ panfrost_init_screen_caps(struct panfrost_screen *screen)
    /* Our GL 3.x implementation is WIP */
    bool is_gl3 = dev->debug & PAN_DBG_GL3;
 
-   /* Native MRT is introduced with v5 */
-   bool has_mrt = (dev->arch >= 5);
+   unsigned max_tib_size =
+      pan_get_max_tib_size(dev->arch, dev->model);
 
    caps->npot_textures = true;
    caps->mixed_color_depth_bits = true;
@@ -501,7 +649,7 @@ panfrost_init_screen_caps(struct panfrost_screen *screen)
                                             : PIPE_POINT_SIZE_LOWER_NEVER;
 
    caps->max_render_targets =
-   caps->fbfetch = has_mrt ? 8 : 1;
+   caps->fbfetch = pan_get_max_cbufs(dev->arch, max_tib_size);
    caps->fbfetch_coherent = true;
 
    caps->max_dual_source_render_targets = 1;
@@ -516,6 +664,8 @@ panfrost_init_screen_caps(struct panfrost_screen *screen)
    /* Compile side is TODO for Midgard. */
    caps->shader_clock = dev->arch >= 6 &&
       dev->kmod.props.gpu_can_query_timestamp;
+   caps->shader_realtime_clock = dev->arch >= 6 &&
+      dev->kmod.props.gpu_can_query_timestamp;
 
    caps->vs_instanceid = true;
    caps->texture_multisample = true;
@@ -636,7 +786,7 @@ panfrost_init_screen_caps(struct panfrost_screen *screen)
 
    caps->shader_buffer_offset_alignment = 4;
 
-   caps->max_varyings = dev->arch >= 9 ? 16 : 32;
+   caps->max_varyings = 32;
 
    /* Removed in v6 (Bifrost) */
    caps->gl_clamp =
@@ -723,7 +873,6 @@ panfrost_destroy_screen(struct pipe_screen *pscreen)
    panfrost_resource_screen_destroy(pscreen);
    panfrost_pool_cleanup(&screen->mempools.bin);
    panfrost_pool_cleanup(&screen->mempools.desc);
-   pan_blend_shader_cache_cleanup(&dev->blend_shaders);
 
    if (screen->vtbl.screen_destroy)
       screen->vtbl.screen_destroy(pscreen);
@@ -736,12 +885,11 @@ panfrost_destroy_screen(struct pipe_screen *pscreen)
    ralloc_free(pscreen);
 }
 
-static const void *
+static const struct nir_shader_compiler_options *
 panfrost_screen_get_compiler_options(struct pipe_screen *pscreen,
-                                     enum pipe_shader_ir ir,
                                      enum pipe_shader_type shader)
 {
-   return pan_screen(pscreen)->vtbl.get_compiler_options();
+   return pan_shader_get_compiler_options(pan_screen(pscreen)->dev.arch);
 }
 
 static struct disk_cache *
@@ -820,6 +968,8 @@ panfrost_create_screen(int fd, const struct pipe_screen_config *config,
    screen->max_afbc_packing_ratio = debug_get_num_option(
       "PAN_MAX_AFBC_PACKING_RATIO", DEFAULT_MAX_AFBC_PACKING_RATIO);
 
+   util_cpu_trace_init();
+
    if (panfrost_open_device(screen, fd, dev)) {
       ralloc_free(screen);
       return NULL;
@@ -828,10 +978,13 @@ panfrost_create_screen(int fd, const struct pipe_screen_config *config,
    if (dev->debug & PAN_DBG_NO_AFBC)
       dev->has_afbc = false;
 
+   dev->relaxed_afbc_yuv_imports =
+      driQueryOptionb(config->options, "pan_relax_afbc_yuv_imports");
+
    /* Bail early on unsupported hardware */
    if (dev->model == NULL) {
       debug_printf("panfrost: Unsupported model %X",
-                   panfrost_device_gpu_id(dev));
+                   panfrost_device_gpu_prod_id(dev));
       panfrost_destroy_screen(&(screen->base));
       return NULL;
    }
@@ -839,10 +992,16 @@ panfrost_create_screen(int fd, const struct pipe_screen_config *config,
    snprintf(screen->renderer_string, sizeof(screen->renderer_string),
             "%s (Panfrost)", dev->model->name);
 
+   screen->afbc_tiled = driQueryOptionb(config->options, "pan_afbc_tiled");
+
    screen->force_afbc_packing = dev->debug & PAN_DBG_FORCE_PACK;
    if (!screen->force_afbc_packing)
       screen->force_afbc_packing = driQueryOptionb(config->options,
                                                    "pan_force_afbc_packing");
+   screen->afbcp_reads_threshold = driQueryOptioni(config->options,
+                                                   "pan_afbcp_reads_threshold");
+   screen->afbcp_gpu_payload_sizes = driQueryOptionb(config->options,
+                                                     "pan_afbcp_gpu_payload_sizes");
 
    const char *option = debug_get_option("PAN_AFRC_RATE", NULL);
    if (!option) {
@@ -901,8 +1060,6 @@ panfrost_create_screen(int fd, const struct pipe_screen_config *config,
       panfrost_query_compression_modifiers;
 
    panfrost_resource_screen_init(&screen->base);
-   pan_blend_shader_cache_init(&dev->blend_shaders,
-                               panfrost_device_gpu_id(dev));
 
    panfrost_init_shader_caps(screen);
    panfrost_init_compute_caps(screen);
