@@ -218,16 +218,12 @@ v3d_init_compute_caps(struct v3d_screen *screen)
         /* GL_MAX_COMPUTE_SHARED_MEMORY_SIZE */
         caps->max_local_size = 32768;
 
-        caps->max_private_size =
-        caps->max_input_size = 4096;
-
         struct sysinfo si;
         sysinfo(&si);
         caps->max_global_size = si.totalram;
         caps->max_mem_alloc_size = MIN2(V3D_MAX_BUFFER_RANGE, si.totalram);
 
         caps->max_compute_units = 1;
-        caps->images_supported = true;
         caps->subgroup_sizes = 16;
 }
 
@@ -338,6 +334,7 @@ v3d_init_screen_caps(struct v3d_screen *screen)
         caps->max_render_targets = V3D_MAX_RENDER_TARGETS(screen->devinfo.ver);
         caps->fbfetch = caps->max_render_targets;
         caps->fbfetch_coherent = true;
+        caps->max_dual_source_render_targets = 1;
 
         caps->vendor_id = 0x14E4;
 
@@ -372,6 +369,7 @@ v3d_init_screen_caps(struct v3d_screen *screen)
 
         caps->native_fence_fd = true;
 
+        caps->clip_planes = 0;
         caps->depth_clip_disable = screen->devinfo.ver >= 71;
 
         caps->min_line_width =
@@ -534,9 +532,8 @@ v3d_screen_is_format_supported(struct pipe_screen *pscreen,
         return true;
 }
 
-static const void *
+static const struct nir_shader_compiler_options *
 v3d_screen_get_compiler_options(struct pipe_screen *pscreen,
-                                enum pipe_shader_ir ir,
                                 enum pipe_shader_type shader)
 {
         struct v3d_screen *screen = v3d_screen(pscreen);
@@ -548,13 +545,14 @@ v3d_screen_get_compiler_options(struct pipe_screen *pscreen,
                 .lower_uadd_sat = true,
                 .lower_usub_sat = true,
                 .lower_iadd_sat = true,
-                .lower_all_io_to_temps = true,
                 .lower_extract_byte = true,
                 .lower_extract_word = true,
                 .lower_insert_byte = true,
                 .lower_insert_word = true,
                 .lower_bitfield_insert = true,
                 .lower_bitfield_extract = true,
+                .lower_bitfield_extract16 = true,
+                .lower_bitfield_extract8 = true,
                 .lower_bitfield_reverse = true,
                 .lower_bit_count = true,
                 .lower_cs_local_id_to_index = true,
@@ -605,17 +603,6 @@ v3d_screen_get_compiler_options(struct pipe_screen *pscreen,
                 .has_uclz = true,
                 .divergence_analysis_options =
                        nir_divergence_multiple_workgroup_per_compute_subgroup,
-                /* We don't currently support this in the backend, but that is
-                 * okay because our NIR compiler sets the option
-                 * lower_all_io_to_temps, which will eliminate indirect
-                 * indexing on all input/output variables by translating it to
-                 * indirect indexing on temporary variables instead, which we
-                 * will then lower to scratch. We prefer this over setting this
-                 * to 0, which would cause if-ladder injection to eliminate
-                 * indirect indexing on inputs.
-                 */
-                .support_indirect_inputs = (uint8_t)BITFIELD_MASK(PIPE_SHADER_TYPES),
-                .support_indirect_outputs = (uint8_t)BITFIELD_MASK(PIPE_SHADER_TYPES),
                 /* This will enable loop unrolling in the state tracker so we won't
                  * be able to selectively disable it in backend if it leads to
                  * lower thread counts or TMU spills. Choose a conservative maximum to
@@ -777,6 +764,8 @@ v3d_screen_create(int fd, const struct pipe_screen_config *config,
         struct v3d_screen *screen = rzalloc(NULL, struct v3d_screen);
         struct pipe_screen *pscreen;
 
+        util_cpu_trace_init();
+
         pscreen = &screen->base;
 
         pscreen->destroy = v3d_screen_destroy;
