@@ -19,9 +19,9 @@ apply_dst_clears(struct zink_context *ctx, const struct pipe_blit_info *info, bo
    if (info->scissor_enable) {
       struct u_rect rect = { info->scissor.minx, info->scissor.maxx,
                              info->scissor.miny, info->scissor.maxy };
-      zink_fb_clears_apply_or_discard(ctx, info->dst.resource, rect, discard_only);
+      zink_fb_clears_apply_or_discard(ctx, info->dst.resource, rect, info->dst.box.z, info->dst.box.depth, discard_only);
    } else
-      zink_fb_clears_apply_or_discard(ctx, info->dst.resource, zink_rect_from_box(&info->dst.box), discard_only);
+      zink_fb_clears_apply_or_discard(ctx, info->dst.resource, zink_rect_from_box(&info->dst.box), info->dst.box.z, info->dst.box.depth, discard_only);
 }
 
 static bool
@@ -31,6 +31,7 @@ blit_resolve(struct zink_context *ctx, const struct pipe_blit_info *info, bool *
        util_format_get_mask(info->src.format) != info->mask ||
        util_format_is_depth_or_stencil(info->dst.format) ||
        info->scissor_enable ||
+       info->swizzle_enable ||
        info->alpha_blend)
       return false;
 
@@ -65,20 +66,25 @@ blit_resolve(struct zink_context *ctx, const struct pipe_blit_info *info, bool *
 
 
    apply_dst_clears(ctx, info, false);
-   zink_fb_clears_apply_region(ctx, info->src.resource, zink_rect_from_box(&info->src.box));
+   zink_fb_clears_apply_region(ctx, info->src.resource, zink_rect_from_box(&info->src.box), info->src.box.z, info->src.box.depth);
 
    if (src->obj->dt)
       *needs_present_readback = zink_kopper_acquire_readback(ctx, src, &use_src);
 
-   struct zink_batch *batch = &ctx->batch;
-   zink_resource_setup_transfer_layouts(ctx, use_src, dst);
+   if (zink_screen(ctx->base.screen)->driver_workarounds.general_layout) {
+      zink_resource_image_transfer_dst_barrier(ctx, dst, info->dst.level, &info->dst.box, false);
+      screen->image_barrier(ctx, use_src,
+                              VK_IMAGE_LAYOUT_GENERAL,
+                              VK_ACCESS_TRANSFER_READ_BIT | VK_ACCESS_TRANSFER_WRITE_BIT,
+                              VK_PIPELINE_STAGE_TRANSFER_BIT);
+   } else {
+      zink_resource_setup_transfer_layouts(ctx, use_src, dst);
+   }
    VkCommandBuffer cmdbuf = *needs_present_readback ?
-                            ctx->batch.state->cmdbuf :
+                            ctx->bs->cmdbuf :
                             zink_get_cmdbuf(ctx, src, dst);
-   if (cmdbuf == ctx->batch.state->cmdbuf)
-      zink_flush_dgc_if_enabled(ctx);
-   zink_batch_reference_resource_rw(batch, use_src, false);
-   zink_batch_reference_resource_rw(batch, dst, true);
+   zink_batch_reference_resource_rw(ctx, use_src, false);
+   zink_batch_reference_resource_rw(ctx, dst, true);
 
    bool marker = zink_cmd_debug_marker_begin(ctx, cmdbuf, "blit_resolve(%s->%s, %dx%d->%dx%d)",
                                              util_format_short_name(info->src.format),
@@ -148,6 +154,7 @@ blit_native(struct zink_context *ctx, const struct pipe_blit_info *info, bool *n
    if (util_format_get_mask(info->dst.format) != info->mask ||
        util_format_get_mask(info->src.format) != info->mask ||
        info->scissor_enable ||
+       info->swizzle_enable ||
        info->alpha_blend)
       return false;
 
@@ -272,20 +279,25 @@ blit_native(struct zink_context *ctx, const struct pipe_blit_info *info, bool *n
    assert(region.dstOffsets[0].z != region.dstOffsets[1].z);
 
    apply_dst_clears(ctx, info, false);
-   zink_fb_clears_apply_region(ctx, info->src.resource, zink_rect_from_box(&info->src.box));
+   zink_fb_clears_apply_region(ctx, info->src.resource, zink_rect_from_box(&info->src.box), info->src.box.z, info->src.box.depth);
 
    if (src->obj->dt)
       *needs_present_readback = zink_kopper_acquire_readback(ctx, src, &use_src);
 
-   struct zink_batch *batch = &ctx->batch;
-   zink_resource_setup_transfer_layouts(ctx, use_src, dst);
+   if (zink_screen(ctx->base.screen)->driver_workarounds.general_layout) {
+      zink_resource_image_transfer_dst_barrier(ctx, dst, info->dst.level, &info->dst.box, false);
+      screen->image_barrier(ctx, use_src,
+                              VK_IMAGE_LAYOUT_GENERAL,
+                              VK_ACCESS_TRANSFER_READ_BIT | VK_ACCESS_TRANSFER_WRITE_BIT,
+                              VK_PIPELINE_STAGE_TRANSFER_BIT);
+   } else {
+      zink_resource_setup_transfer_layouts(ctx, use_src, dst);
+   }
    VkCommandBuffer cmdbuf = *needs_present_readback ?
-                            ctx->batch.state->cmdbuf :
+                            ctx->bs->cmdbuf :
                             zink_get_cmdbuf(ctx, src, dst);
-   if (cmdbuf == ctx->batch.state->cmdbuf)
-      zink_flush_dgc_if_enabled(ctx);
-   zink_batch_reference_resource_rw(batch, use_src, false);
-   zink_batch_reference_resource_rw(batch, dst, true);
+   zink_batch_reference_resource_rw(ctx, use_src, false);
+   zink_batch_reference_resource_rw(ctx, dst, true);
 
    bool marker = zink_cmd_debug_marker_begin(ctx, cmdbuf, "blit_native(%s->%s, %dx%d->%dx%d)",
                                              util_format_short_name(info->src.format),
@@ -334,6 +346,19 @@ zink_blit(struct pipe_context *pctx,
    struct zink_resource *use_src = src;
    struct zink_resource *dst = zink_resource(info->dst.resource);
    bool needs_present_readback = false;
+
+   if (ctx->awaiting_resolve && ctx->in_rp && ctx->dynamic_fb.tc_info.has_resolve) {
+      bool is_depth = util_format_is_depth_or_stencil(info->src.format);
+      struct pipe_resource *resolve = ctx->fb_state.resolve;
+      if (!resolve)
+         resolve = ctx->dynamic_fb.tc_info.resolve[is_depth];
+      if (resolve == info->dst.resource) {
+         zink_batch_no_rp_safe(ctx);
+         ctx->awaiting_resolve = false;
+         return;
+      }
+   }
+
    if (zink_is_swapchain(dst)) {
       if (!zink_kopper_acquire(ctx, dst, UINT64_MAX))
          return;
@@ -367,7 +392,7 @@ zink_blit(struct pipe_context *pctx,
             depth_blit.mask = PIPE_MASK_Z;
             if (util_blitter_is_blit_supported(ctx->blitter, &depth_blit)) {
                zink_blit_begin(ctx, ZINK_BLIT_SAVE_FB | ZINK_BLIT_SAVE_FS | ZINK_BLIT_SAVE_TEXTURES);
-               util_blitter_blit(ctx->blitter, &depth_blit);
+               util_blitter_blit(ctx->blitter, &depth_blit, NULL);
             } else {
                mesa_loge("ZINK: depth blit unsupported %s -> %s",
                          util_format_short_name(info->src.resource->format),
@@ -386,7 +411,7 @@ zink_blit(struct pipe_context *pctx,
    }
 
    if (src->obj->dt) {
-      zink_fb_clears_apply_region(ctx, info->src.resource, zink_rect_from_box(&info->src.box));
+      zink_fb_clears_apply_region(ctx, info->src.resource, zink_rect_from_box(&info->src.box), info->src.box.z, info->src.box.depth);
       needs_present_readback = zink_kopper_acquire_readback(ctx, src, &use_src);
    }
 
@@ -394,7 +419,7 @@ zink_blit(struct pipe_context *pctx,
     * flush all pending clears anyway
     */
    apply_dst_clears(ctx, info, true);
-   zink_fb_clears_apply_region(ctx, info->src.resource, zink_rect_from_box(&info->src.box));
+   zink_fb_clears_apply_region(ctx, info->src.resource, zink_rect_from_box(&info->src.box), info->src.box.z, info->src.box.depth);
    unsigned rp_clears_enabled = ctx->rp_clears_enabled;
    unsigned clears_enabled = ctx->clears_enabled;
    if (!dst->fb_bind_count) {
@@ -419,26 +444,23 @@ zink_blit(struct pipe_context *pctx,
    if (whole)
       pctx->invalidate_resource(pctx, info->dst.resource);
 
-   zink_flush_dgc_if_enabled(ctx);
    ctx->unordered_blitting = !(info->render_condition_enable && ctx->render_condition_active) &&
-                             zink_screen(ctx->base.screen)->info.have_KHR_dynamic_rendering &&
                              !needs_present_readback &&
-                             zink_get_cmdbuf(ctx, src, dst) == ctx->batch.state->reordered_cmdbuf;
-   VkCommandBuffer cmdbuf = ctx->batch.state->cmdbuf;
+                             zink_get_cmdbuf(ctx, src, dst) == ctx->bs->reordered_cmdbuf;
+   VkCommandBuffer cmdbuf = ctx->bs->cmdbuf;
    VkPipeline pipeline = ctx->gfx_pipeline_state.pipeline;
-   bool in_rp = ctx->batch.in_rp;
+   bool in_rp = ctx->in_rp;
    uint64_t tc_data = ctx->dynamic_fb.tc_info.data;
    bool queries_disabled = ctx->queries_disabled;
-   bool rp_changed = ctx->rp_changed || (!ctx->fb_state.zsbuf && util_format_is_depth_or_stencil(info->dst.format));
+   bool rp_changed = ctx->rp_changed || (!ctx->fb_state.zsbuf.texture && util_format_is_depth_or_stencil(info->dst.format));
    unsigned ds3_states = ctx->ds3_states;
    bool rp_tc_info_updated = ctx->rp_tc_info_updated;
    if (ctx->unordered_blitting) {
       /* for unordered blit, swap the unordered cmdbuf for the main one for the whole op to avoid conditional hell */
-      ctx->batch.state->cmdbuf = ctx->batch.state->reordered_cmdbuf;
-      ctx->batch.in_rp = false;
+      ctx->bs->cmdbuf = ctx->bs->reordered_cmdbuf;
+      ctx->in_rp = false;
       ctx->rp_changed = true;
       ctx->queries_disabled = true;
-      ctx->batch.state->has_barriers = true;
       ctx->pipeline_changed[0] = true;
       zink_reset_ds3_states(ctx);
       zink_select_draw_vbo(ctx);
@@ -450,13 +472,14 @@ zink_blit(struct pipe_context *pctx,
       zink_resource_object_init_mutable(ctx, dst);
    zink_blit_barriers(ctx, use_src, dst, whole);
    ctx->blitting = true;
+   ctx->blit_scissor = info->scissor_enable;
+   ctx->blit_nearest = info->filter == PIPE_TEX_FILTER_NEAREST;
 
    if (stencil_blit) {
-      struct pipe_surface *dst_view, dst_templ;
+      struct pipe_surface dst_templ;
       util_blitter_default_dst_texture(&dst_templ, info->dst.resource, info->dst.level, info->dst.box.z);
-      dst_view = pctx->create_surface(pctx, info->dst.resource, &dst_templ);
 
-      util_blitter_clear_depth_stencil(ctx->blitter, dst_view, PIPE_CLEAR_STENCIL,
+      util_blitter_clear_depth_stencil(ctx->blitter, &dst_templ, PIPE_CLEAR_STENCIL,
                                        0, 0, info->dst.box.x, info->dst.box.y,
                                        info->dst.box.width, info->dst.box.height);
       zink_blit_begin(ctx, ZINK_BLIT_SAVE_FB | ZINK_BLIT_SAVE_FS | ZINK_BLIT_SAVE_TEXTURES | ZINK_BLIT_SAVE_FS_CONST_BUF);
@@ -468,25 +491,23 @@ zink_blit(struct pipe_context *pctx,
                                     info->src.level,
                                     &info->src.box,
                                     info->scissor_enable ? &info->scissor : NULL);
-
-      pipe_surface_release(pctx, &dst_view);
    } else {
       struct pipe_blit_info new_info = *info;
       new_info.src.resource = &use_src->base.b;
-      util_blitter_blit(ctx->blitter, &new_info);
+      util_blitter_blit(ctx->blitter, &new_info, NULL);
    }
    ctx->blitting = false;
    ctx->rp_clears_enabled = rp_clears_enabled;
    ctx->clears_enabled = clears_enabled;
    if (ctx->unordered_blitting) {
       zink_batch_no_rp(ctx);
-      ctx->batch.in_rp = in_rp;
+      ctx->in_rp = in_rp;
       ctx->gfx_pipeline_state.rp_state = zink_update_rendering_info(ctx);
       ctx->rp_changed = rp_changed;
       ctx->rp_tc_info_updated |= rp_tc_info_updated;
       ctx->queries_disabled = queries_disabled;
       ctx->dynamic_fb.tc_info.data = tc_data;
-      ctx->batch.state->cmdbuf = cmdbuf;
+      ctx->bs->cmdbuf = cmdbuf;
       ctx->gfx_pipeline_state.pipeline = pipeline;
       ctx->pipeline_changed[0] = true;
       ctx->ds3_states = ds3_states;
@@ -515,7 +536,7 @@ zink_blit_begin(struct zink_context *ctx, enum zink_blit_flags flags)
    util_blitter_save_tesseval_shader(ctx->blitter, ctx->gfx_stages[MESA_SHADER_TESS_EVAL]);
    util_blitter_save_geometry_shader(ctx->blitter, ctx->gfx_stages[MESA_SHADER_GEOMETRY]);
    util_blitter_save_rasterizer(ctx->blitter, ctx->rast_state);
-   util_blitter_save_so_targets(ctx->blitter, ctx->num_so_targets, ctx->so_targets);
+   util_blitter_save_so_targets(ctx->blitter, ctx->num_so_targets, ctx->so_targets, MESA_PRIM_UNKNOWN);
 
    if (flags & ZINK_BLIT_SAVE_FS_CONST_BUF)
       util_blitter_save_fragment_constant_buffer_slot(ctx->blitter, ctx->ubos[MESA_SHADER_FRAGMENT]);
@@ -574,14 +595,14 @@ zink_blit_barriers(struct zink_context *ctx, struct zink_resource *src, struct z
       pipeline = VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT;
    }
    if (src == dst) {
-      VkImageLayout layout = zink_screen(ctx->base.screen)->info.have_EXT_attachment_feedback_loop_layout ?
+      VkImageLayout layout = !screen->driver_workarounds.general_layout && screen->info.have_EXT_attachment_feedback_loop_layout ?
                              VK_IMAGE_LAYOUT_ATTACHMENT_FEEDBACK_LOOP_OPTIMAL_EXT :
                              VK_IMAGE_LAYOUT_GENERAL;
       screen->image_barrier(ctx, src, layout, VK_ACCESS_SHADER_READ_BIT | flags, VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT | pipeline);
    } else {
       if (src) {
-         VkImageLayout layout = util_format_is_depth_or_stencil(src->base.b.format) &&
-                                src->obj->vkusage & VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT ?
+         VkImageLayout layout = screen->driver_workarounds.general_layout ? VK_IMAGE_LAYOUT_GENERAL :
+                                util_format_is_depth_or_stencil(src->base.b.format) && src->obj->vkusage & VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT ?
                                 VK_IMAGE_LAYOUT_DEPTH_STENCIL_READ_ONLY_OPTIMAL :
                                 VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL;
          screen->image_barrier(ctx, src, layout,
@@ -589,9 +610,10 @@ zink_blit_barriers(struct zink_context *ctx, struct zink_resource *src, struct z
          if (!ctx->unordered_blitting)
             src->obj->unordered_read = false;
       }
-      VkImageLayout layout = util_format_is_depth_or_stencil(dst->base.b.format) ?
-                           VK_IMAGE_LAYOUT_DEPTH_STENCIL_ATTACHMENT_OPTIMAL :
-                           VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL;
+      VkImageLayout layout = screen->driver_workarounds.general_layout ? VK_IMAGE_LAYOUT_GENERAL :
+                             util_format_is_depth_or_stencil(dst->base.b.format) ?
+                             VK_IMAGE_LAYOUT_DEPTH_STENCIL_ATTACHMENT_OPTIMAL :
+                             VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL;
       screen->image_barrier(ctx, dst, layout, flags, pipeline);
    }
    if (!ctx->unordered_blitting)
@@ -644,3 +666,36 @@ zink_blit_region_covers(struct u_rect region, struct u_rect covers)
     return intersect.x0 == c.x0 && intersect.y0 == c.y0 &&
            intersect.x1 == c.x1 && intersect.y1 == c.y1;
 }
+
+void
+zink_draw_rectangle(struct blitter_context *blitter, void *vertex_elements_cso,
+                    blitter_get_vs_func get_vs, int x1, int y1, int x2, int y2,
+                    float depth, unsigned num_instances, enum blitter_attrib_type type,
+                    const struct blitter_attrib *attrib)
+{
+   struct zink_context *ctx = zink_context(blitter->pipe);
+
+   struct blitter_attrib new_attrib = *attrib;
+
+   /* Avoid inconsistencies in rounding between both triangles which can show with
+    * nearest filtering by expanding the rect so only one triangle is effectively drawn.
+    */
+   if (ctx->blit_scissor && ctx->blit_nearest) {
+      int64_t new_x1 = (int64_t)x1 * 2 - x2;
+      int64_t new_y2 = (int64_t)y2 * 2 - y1;
+      if (new_x1 < INT32_MAX && new_x1 > INT32_MIN &&
+          new_y2 < INT32_MAX && new_y2 > INT32_MIN) {
+         x1 = new_x1;
+         y2 = new_y2;
+
+         if (type == UTIL_BLITTER_ATTRIB_TEXCOORD_XY ||
+             type == UTIL_BLITTER_ATTRIB_TEXCOORD_XYZW) {
+            new_attrib.texcoord.x1 += new_attrib.texcoord.x1 - new_attrib.texcoord.x2;
+            new_attrib.texcoord.y2 += new_attrib.texcoord.y2 - new_attrib.texcoord.y1;
+         }
+      }
+   }
+
+   util_blitter_draw_rectangle(blitter, vertex_elements_cso, get_vs, x1, y1, x2, y2,
+                               depth, num_instances, type, &new_attrib);
+}
