@@ -202,10 +202,9 @@ descriptor_util_pool_key_get(struct zink_screen *screen, enum zink_descriptor_ty
       hash = hash_descriptor_pool_key(&key);
       simple_mtx_lock(&screen->desc_pool_keys_lock);
       struct set_entry *he = _mesa_set_search_pre_hashed(&screen->desc_pool_keys[type], hash, &key);
-      if (he) {
-         simple_mtx_unlock(&screen->desc_pool_keys_lock);
+      simple_mtx_unlock(&screen->desc_pool_keys_lock);
+      if (he)
          return (void*)he->key;
-      }
    }
 
    struct zink_descriptor_pool_key *pool_key = rzalloc(screen, struct zink_descriptor_pool_key);
@@ -214,6 +213,7 @@ descriptor_util_pool_key_get(struct zink_screen *screen, enum zink_descriptor_ty
    assert(pool_key->num_type_sizes);
    memcpy(pool_key->sizes, sizes, num_type_sizes * sizeof(VkDescriptorPoolSize));
    if (type != ZINK_DESCRIPTOR_TYPE_UNIFORMS) {
+      simple_mtx_lock(&screen->desc_pool_keys_lock);
       _mesa_set_add_pre_hashed(&screen->desc_pool_keys[type], hash, pool_key);
       pool_key->id = screen->desc_pool_keys[type].entries - 1;
       simple_mtx_unlock(&screen->desc_pool_keys_lock);
@@ -273,6 +273,8 @@ zink_descriptor_util_push_layouts_get(struct zink_context *ctx, struct zink_desc
 VkImageLayout
 zink_descriptor_util_image_layout_eval(const struct zink_context *ctx, const struct zink_resource *res, bool is_compute)
 {
+   if (zink_screen(ctx->base.screen)->driver_workarounds.general_layout)
+      return VK_IMAGE_LAYOUT_GENERAL;
    if (res->bindless[0] || res->bindless[1]) {
       /* bindless needs most permissive layout */
       if (res->image_bind_count[0] || res->image_bind_count[1])
@@ -866,15 +868,9 @@ set_pool(struct zink_batch_state *bs, struct zink_program *pg, struct zink_descr
    assert(type != ZINK_DESCRIPTOR_TYPE_UNIFORMS);
    assert(mpool);
    const struct zink_descriptor_pool_key *pool_key = pg->dd.pool_key[type];
-   size_t size = bs->dd.pools[type].capacity;
    /* ensure the pool array is big enough to have an element for this key */
-   if (!util_dynarray_resize(&bs->dd.pools[type], struct zink_descriptor_pool_multi*, pool_key->id + 1))
+   if (!util_dynarray_resize_zero(&bs->dd.pools[type], struct zink_descriptor_pool_multi*, pool_key->id + 1))
       return false;
-   if (size != bs->dd.pools[type].capacity) {
-      /* when resizing, always zero the new data to avoid garbage */
-      uint8_t *data = bs->dd.pools[type].data;
-      memset(data + size, 0, bs->dd.pools[type].capacity - size);
-   }
    /* dynarray can't track sparse array sizing, so the array size must be manually tracked */
    bs->dd.pool_size[type] = MAX2(bs->dd.pool_size[type], pool_key->id + 1);
    struct zink_descriptor_pool_multi **mppool = util_dynarray_element(&bs->dd.pools[type], struct zink_descriptor_pool_multi*, pool_key->id);
