@@ -23,7 +23,7 @@ use std::os::raw::c_void;
 use std::ptr;
 use std::sync::Arc;
 
-type CLGLMappings = Option<HashMap<Arc<PipeResource>, Arc<PipeResource>>>;
+type CLGLMappings = Option<HashMap<PipeResource, PipeResource>>;
 
 pub struct XPlatManager {
     #[cfg(glx)]
@@ -206,60 +206,6 @@ impl GLCtxManager {
         }
     }
 
-    fn do_flush(&self, exports_in: &mut [mesa_glinterop_export_in]) -> CLResult<Option<FenceFd>> {
-        let mut fd = -1;
-        let mut flush_out = mesa_glinterop_flush_out {
-            version: 1,
-            fence_fd: &mut fd,
-            ..Default::default()
-        };
-
-        let err = match self.gl_ctx {
-            GLCtx::EGL(disp, ctx) => {
-                let egl_flush_objects_func = self
-                    .xplat_manager
-                    .MesaGLInteropEGLFlushObjects()?
-                    .ok_or(CL_INVALID_GL_SHAREGROUP_REFERENCE_KHR)?;
-
-                unsafe {
-                    egl_flush_objects_func(
-                        disp.cast(),
-                        ctx.cast(),
-                        exports_in.len() as u32,
-                        exports_in.as_mut_ptr(),
-                        &mut flush_out,
-                    )
-                }
-            }
-            GLCtx::GLX(disp, ctx) => {
-                let glx_flush_objects_func = self
-                    .xplat_manager
-                    .MesaGLInteropGLXFlushObjects()?
-                    .ok_or(CL_INVALID_GL_SHAREGROUP_REFERENCE_KHR)?;
-
-                unsafe {
-                    glx_flush_objects_func(
-                        disp.cast(),
-                        ctx.cast(),
-                        exports_in.len() as u32,
-                        exports_in.as_mut_ptr(),
-                        &mut flush_out,
-                    )
-                }
-            }
-        };
-
-        if err != 0 {
-            return Err(interop_to_cl_error(err));
-        }
-
-        if fd != -1 {
-            Ok(Some(FenceFd { fd: fd }))
-        } else {
-            Ok(None)
-        }
-    }
-
     pub fn export_object(
         &self,
         cl_ctx: &Arc<Context>,
@@ -283,12 +229,13 @@ impl GLCtxManager {
             ..Default::default()
         };
 
-        if let Some(fence_fd) = self.do_flush(&mut [export_in])? {
-            cl_ctx.devs.iter().for_each(|dev| {
-                let fence = dev.helper_ctx().import_fence(&fence_fd);
-                fence.wait();
-            });
-        }
+        let mut fd = -1;
+
+        let mut flush_out = mesa_glinterop_flush_out {
+            version: 1,
+            fence_fd: &mut fd,
+            ..Default::default()
+        };
 
         let err = unsafe {
             match &self.gl_ctx {
@@ -297,14 +244,68 @@ impl GLCtxManager {
                         .MesaGLInteropEGLExportObject()?
                         .ok_or(CL_INVALID_GL_SHAREGROUP_REFERENCE_KHR)?;
 
-                    egl_export_object_func(disp.cast(), ctx.cast(), &mut export_in, &mut export_out)
+                    let egl_flush_objects_func = xplat_manager
+                        .MesaGLInteropEGLFlushObjects()?
+                        .ok_or(CL_INVALID_GL_SHAREGROUP_REFERENCE_KHR)?;
+
+                    let err_flush = egl_flush_objects_func(
+                        disp.cast(),
+                        ctx.cast(),
+                        1,
+                        &mut export_in,
+                        &mut flush_out,
+                    );
+                    // TODO: use fence_server_sync in ctx inside the queue thread
+                    let fence_fd = FenceFd { fd };
+                    cl_ctx.devs.iter().for_each(|dev| {
+                        let fence = dev.helper_ctx().import_fence(&fence_fd);
+                        fence.wait();
+                    });
+
+                    if err_flush != 0 {
+                        err_flush
+                    } else {
+                        egl_export_object_func(
+                            disp.cast(),
+                            ctx.cast(),
+                            &mut export_in,
+                            &mut export_out,
+                        )
+                    }
                 }
                 GLCtx::GLX(disp, ctx) => {
                     let glx_export_object_func = xplat_manager
                         .MesaGLInteropGLXExportObject()?
                         .ok_or(CL_INVALID_GL_SHAREGROUP_REFERENCE_KHR)?;
 
-                    glx_export_object_func(disp.cast(), ctx.cast(), &mut export_in, &mut export_out)
+                    let glx_flush_objects_func = xplat_manager
+                        .MesaGLInteropGLXFlushObjects()?
+                        .ok_or(CL_INVALID_GL_SHAREGROUP_REFERENCE_KHR)?;
+
+                    let err_flush = glx_flush_objects_func(
+                        disp.cast(),
+                        ctx.cast(),
+                        1,
+                        &mut export_in,
+                        &mut flush_out,
+                    );
+                    // TODO: use fence_server_sync in ctx inside the queue thread
+                    let fence_fd = FenceFd { fd };
+                    cl_ctx.devs.iter().for_each(|dev| {
+                        let fence = dev.helper_ctx().import_fence(&fence_fd);
+                        fence.wait();
+                    });
+
+                    if err_flush != 0 {
+                        err_flush
+                    } else {
+                        glx_export_object_func(
+                            disp.cast(),
+                            ctx.cast(),
+                            &mut export_in,
+                            &mut export_out,
+                        )
+                    }
                 }
             }
         };
@@ -324,16 +325,6 @@ impl GLCtxManager {
             export_out: export_out,
         })
     }
-
-    pub fn flush(&self, mem_objects: &[Mem]) -> CLResult<Option<FenceFd>> {
-        let mut exports_in = mem_objects
-            .iter()
-            .filter_map(|m| m.gl_obj.as_ref())
-            .map(|gl_obj| gl_obj.props)
-            .collect::<Vec<_>>();
-
-        self.do_flush(&mut exports_in)
-    }
 }
 
 #[derive(Clone)]
@@ -428,53 +419,16 @@ impl Drop for GLExportManager {
 }
 
 pub struct GLObject {
-    pub props: mesa_glinterop_export_in,
+    pub gl_object_target: cl_GLenum,
+    pub gl_object_type: cl_gl_object_type,
+    pub gl_object_name: cl_GLuint,
     pub shadow_map: CLGLMappings,
 }
 
-// SAFETY: We never use the pointer inside mesa_glinterop_export_in for anything.
-unsafe impl Send for GLObject {}
-unsafe impl Sync for GLObject {}
-
-impl GLObject {
-    pub fn cl_gl_type(&self) -> CLResult<cl_gl_object_type> {
-        Ok(match self.props.target {
-            GL_ARRAY_BUFFER => CL_GL_OBJECT_BUFFER,
-            GL_TEXTURE_BUFFER => CL_GL_OBJECT_TEXTURE_BUFFER,
-            GL_RENDERBUFFER => CL_GL_OBJECT_RENDERBUFFER,
-            GL_TEXTURE_1D => CL_GL_OBJECT_TEXTURE1D,
-            GL_TEXTURE_1D_ARRAY => CL_GL_OBJECT_TEXTURE1D_ARRAY,
-            GL_TEXTURE_CUBE_MAP_NEGATIVE_X
-            | GL_TEXTURE_CUBE_MAP_NEGATIVE_Y
-            | GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
-            | GL_TEXTURE_CUBE_MAP_POSITIVE_X
-            | GL_TEXTURE_CUBE_MAP_POSITIVE_Y
-            | GL_TEXTURE_CUBE_MAP_POSITIVE_Z
-            | GL_TEXTURE_2D
-            | GL_TEXTURE_RECTANGLE => CL_GL_OBJECT_TEXTURE2D,
-            GL_TEXTURE_2D_ARRAY => CL_GL_OBJECT_TEXTURE2D_ARRAY,
-            GL_TEXTURE_3D => CL_GL_OBJECT_TEXTURE3D,
-            _ => return Err(CL_INVALID_VALUE),
-        })
-    }
-
-    pub fn cl_mem_type(&self) -> CLResult<cl_mem_object_type> {
-        mem_type_from_gl(self.props.target)
-    }
-
-    pub fn name(&self) -> cl_GLuint {
-        self.props.obj
-    }
-
-    pub fn target(&self) -> cl_GLuint {
-        self.props.target
-    }
-}
-
 pub fn create_shadow_slice(
-    cube_map: &HashMap<&'static Device, Arc<PipeResource>>,
+    cube_map: &HashMap<&'static Device, PipeResource>,
     image_format: cl_image_format,
-) -> CLResult<HashMap<&'static Device, Arc<PipeResource>>> {
+) -> CLResult<HashMap<&'static Device, PipeResource>> {
     let mut slice = HashMap::new();
 
     for (dev, imported_gl_res) in cube_map {
@@ -495,7 +449,7 @@ pub fn create_shadow_slice(
             )
             .ok_or(CL_OUT_OF_HOST_MEMORY)?;
 
-        slice.insert(*dev, Arc::new(shadow));
+        slice.insert(*dev, shadow);
     }
 
     Ok(slice)
@@ -507,14 +461,14 @@ pub fn copy_cube_to_slice(ctx: &QueueContext, mem_objects: &[Mem]) -> CLResult<(
             continue;
         };
         let gl_obj = image.gl_obj.as_ref().unwrap();
-        if !is_cube_map_face(gl_obj.target()) {
+        if !is_cube_map_face(gl_obj.gl_object_target) {
             continue;
         }
         let width = image.image_desc.image_width;
         let height = image.image_desc.image_height;
 
         // Fill in values for doing the copy
-        let idx = get_array_slice_idx(gl_obj.target());
+        let idx = get_array_slice_idx(gl_obj.gl_object_target);
         let src_origin = CLVec::<usize>::new([0, 0, idx]);
         let dst_offset: [u32; 3] = [0, 0, 0];
         let region = CLVec::<usize>::new([width, height, 1]);
@@ -523,7 +477,7 @@ pub fn copy_cube_to_slice(ctx: &QueueContext, mem_objects: &[Mem]) -> CLResult<(
         let cl_res = image.get_res_for_access(ctx, RWFlags::WR)?;
         let gl_res = gl_obj.shadow_map.as_ref().unwrap().get(cl_res).unwrap();
 
-        ctx.resource_copy_texture(gl_res.as_ref(), cl_res.as_ref(), &dst_offset, &src_bx);
+        ctx.resource_copy_texture(gl_res, cl_res, &dst_offset, &src_bx);
     }
 
     Ok(())
@@ -535,14 +489,14 @@ pub fn copy_slice_to_cube(ctx: &QueueContext, mem_objects: &[Mem]) -> CLResult<(
             continue;
         };
         let gl_obj = image.gl_obj.as_ref().unwrap();
-        if !is_cube_map_face(gl_obj.target()) {
+        if !is_cube_map_face(gl_obj.gl_object_target) {
             continue;
         }
         let width = image.image_desc.image_width;
         let height = image.image_desc.image_height;
 
         // Fill in values for doing the copy
-        let idx = get_array_slice_idx(gl_obj.target()) as u32;
+        let idx = get_array_slice_idx(gl_obj.gl_object_target) as u32;
         let src_origin = CLVec::<usize>::new([0, 0, 0]);
         let dst_offset: [u32; 3] = [0, 0, idx];
         let region = CLVec::<usize>::new([width, height, 1]);
@@ -551,7 +505,7 @@ pub fn copy_slice_to_cube(ctx: &QueueContext, mem_objects: &[Mem]) -> CLResult<(
         let cl_res = image.get_res_for_access(ctx, RWFlags::WR)?;
         let gl_res = gl_obj.shadow_map.as_ref().unwrap().get(cl_res).unwrap();
 
-        ctx.resource_copy_texture(cl_res.as_ref(), gl_res.as_ref(), &dst_offset, &src_bx);
+        ctx.resource_copy_texture(cl_res, gl_res, &dst_offset, &src_bx);
     }
 
     Ok(())
@@ -580,12 +534,15 @@ pub fn cl_to_interop_flags(flags: u32) -> u32 {
     }
 }
 
-pub fn mem_type_from_gl(target: u32) -> CLResult<cl_mem_object_type> {
+pub fn target_from_gl(target: u32) -> CLResult<(u32, u32)> {
+    // CL_INVALID_IMAGE_FORMAT_DESCRIPTOR if the OpenGL texture
+    // internal format does not map to a supported OpenCL image format.
     Ok(match target {
-        GL_ARRAY_BUFFER => CL_MEM_OBJECT_BUFFER,
-        GL_TEXTURE_BUFFER => CL_MEM_OBJECT_IMAGE1D_BUFFER,
-        GL_TEXTURE_1D => CL_MEM_OBJECT_IMAGE1D,
-        GL_TEXTURE_1D_ARRAY => CL_MEM_OBJECT_IMAGE1D_ARRAY,
+        GL_ARRAY_BUFFER => (CL_MEM_OBJECT_BUFFER, CL_GL_OBJECT_BUFFER),
+        GL_TEXTURE_BUFFER => (CL_MEM_OBJECT_IMAGE1D_BUFFER, CL_GL_OBJECT_TEXTURE_BUFFER),
+        GL_RENDERBUFFER => (CL_MEM_OBJECT_IMAGE2D, CL_GL_OBJECT_RENDERBUFFER),
+        GL_TEXTURE_1D => (CL_MEM_OBJECT_IMAGE1D, CL_GL_OBJECT_TEXTURE1D),
+        GL_TEXTURE_1D_ARRAY => (CL_MEM_OBJECT_IMAGE1D_ARRAY, CL_GL_OBJECT_TEXTURE1D_ARRAY),
         GL_TEXTURE_CUBE_MAP_NEGATIVE_X
         | GL_TEXTURE_CUBE_MAP_NEGATIVE_Y
         | GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
@@ -593,10 +550,9 @@ pub fn mem_type_from_gl(target: u32) -> CLResult<cl_mem_object_type> {
         | GL_TEXTURE_CUBE_MAP_POSITIVE_Y
         | GL_TEXTURE_CUBE_MAP_POSITIVE_Z
         | GL_TEXTURE_2D
-        | GL_TEXTURE_RECTANGLE
-        | GL_RENDERBUFFER => CL_MEM_OBJECT_IMAGE2D,
-        GL_TEXTURE_2D_ARRAY => CL_MEM_OBJECT_IMAGE2D_ARRAY,
-        GL_TEXTURE_3D => CL_MEM_OBJECT_IMAGE3D,
+        | GL_TEXTURE_RECTANGLE => (CL_MEM_OBJECT_IMAGE2D, CL_GL_OBJECT_TEXTURE2D),
+        GL_TEXTURE_2D_ARRAY => (CL_MEM_OBJECT_IMAGE2D_ARRAY, CL_GL_OBJECT_TEXTURE2D_ARRAY),
+        GL_TEXTURE_3D => (CL_MEM_OBJECT_IMAGE3D, CL_GL_OBJECT_TEXTURE3D),
         _ => return Err(CL_INVALID_VALUE),
     })
 }
