@@ -3,12 +3,13 @@ use mesa_rust_gen::*;
 use std::{
     marker::PhantomData,
     mem,
+    num::NonZeroU64,
     ptr::{self, NonNull},
 };
 
 use super::context::PipeContext;
 
-#[derive(PartialEq, Eq, Hash)]
+#[derive(Debug, PartialEq, Eq, Hash)]
 #[repr(transparent)]
 pub struct PipeResource {
     pipe: NonNull<pipe_resource>,
@@ -83,6 +84,19 @@ impl PipeResource {
         Some(Self { pipe: res })
     }
 
+    /// Creates a new reference to the underlying GPU resource
+    pub fn new_ref(&self) -> Self {
+        let mut ptr = ptr::null_mut();
+        // SAFETY: pipe_resource_reference will copy the ptr from src to dest, therefore ptr can't
+        //         be NULL.
+        unsafe {
+            pipe_resource_reference(&mut ptr, self.pipe.as_ptr());
+            Self {
+                pipe: NonNull::new_unchecked(ptr),
+            }
+        }
+    }
+
     pub(super) fn pipe(&self) -> *mut pipe_resource {
         self.pipe.as_ptr()
     }
@@ -124,6 +138,14 @@ impl PipeResource {
         self.as_ref().flags & PIPE_RESOURCE_FLAG_RUSTICL_IS_USER != 0
     }
 
+    pub fn resource_get_address(&self) -> Option<NonZeroU64> {
+        let screen_ptr = self.as_ref().screen;
+        // SAFETY: it's a valid pointer and everything.
+        let screen = unsafe { screen_ptr.as_ref().unwrap_unchecked() };
+
+        NonZeroU64::new(unsafe { screen.resource_get_address?(screen_ptr, self.pipe()) })
+    }
+
     pub fn pipe_image_view(&self, read_write: bool) -> PipeImageView {
         debug_assert!(!self.is_buffer());
 
@@ -275,7 +297,9 @@ impl PipeResource {
 
 impl Drop for PipeResource {
     fn drop(&mut self) {
-        unsafe { pipe_resource_reference(&mut self.pipe.as_ptr(), ptr::null_mut()) }
+        unsafe {
+            pipe_resource_reference(&mut self.pipe.as_ptr(), ptr::null_mut());
+        }
     }
 }
 
@@ -326,7 +350,7 @@ impl<'c, 'r> PipeSamplerView<'c, 'r> {
 impl Drop for PipeSamplerView<'_, '_> {
     fn drop(&mut self) {
         unsafe {
-            pipe_sampler_view_release(self.view.as_ptr());
+            pipe_sampler_view_reference(&mut self.view.as_ptr(), ptr::null_mut());
         }
     }
 }
