@@ -1,6 +1,9 @@
 use std::{
     alloc::Layout,
-    collections::{btree_map::Entry, BTreeMap},
+    collections::{
+        btree_map::{Entry, Values, ValuesMut},
+        BTreeMap,
+    },
     hash::{Hash, Hasher},
     mem,
     ops::{Add, Deref},
@@ -95,6 +98,8 @@ impl<T> CheckedPtr<T> for *mut T {
     }
 }
 
+// While std::mem::offset_of!() is stable from 1.77.0, support for nested fields
+// (required in some rusticl cases) wasn't stabilized until 1.82.0.
 // from https://internals.rust-lang.org/t/discussion-on-offset-of/7440/2
 #[macro_export]
 macro_rules! offset_of {
@@ -115,7 +120,7 @@ macro_rules! offset_of {
     }};
 }
 
-// Adapted from libstd since std::ptr::is_aligned is still unstable
+// Adapted from libstd since std::ptr::is_aligned isn't stable until 1.79.0
 // See https://github.com/rust-lang/rust/issues/96284
 #[must_use]
 #[inline]
@@ -123,11 +128,18 @@ pub fn is_aligned<T>(ptr: *const T) -> bool
 where
     T: Sized,
 {
-    let align = mem::align_of::<T>();
+    is_aligned_to(ptr, mem::align_of::<T>())
+}
+
+// Adapted from libstd since std::ptr::is_aligned_to is still unstable
+// See https://github.com/rust-lang/rust/issues/96284
+#[must_use]
+#[inline]
+pub fn is_aligned_to<T>(ptr: *const T, align: usize) -> bool {
     addr(ptr) & (align - 1) == 0
 }
 
-// Adapted from libstd since std::ptr::addr is still unstable
+// Adapted from libstd since std::ptr::addr isn't stable until 1.84.0
 // See https://github.com/rust-lang/rust/issues/95228
 #[must_use]
 #[inline(always)]
@@ -166,6 +178,14 @@ impl<P, T: AllocSize<P>> TrackedPointers<P, T> {
             ptrs: BTreeMap::new(),
         }
     }
+
+    pub fn values(&self) -> Values<'_, P, T> {
+        self.ptrs.values()
+    }
+
+    pub fn values_mut(&mut self) -> ValuesMut<'_, P, T> {
+        self.ptrs.values_mut()
+    }
 }
 
 impl<P, T: AllocSize<P>> TrackedPointers<P, T>
@@ -176,7 +196,7 @@ where
         self.ptrs.contains_key(&ptr)
     }
 
-    pub fn entry(&mut self, ptr: P) -> Entry<'_, P, T> {
+    pub fn entry(&mut self, ptr: P) -> Entry<P, T> {
         self.ptrs.entry(ptr)
     }
 
@@ -192,6 +212,18 @@ where
         None
     }
 
+    pub fn find_alloc_mut(&mut self, ptr: P) -> Option<(P, &mut T)> {
+        if let Some((&base, val)) = self.ptrs.range_mut(..=ptr).next_back() {
+            let size = val.size();
+            // we check if ptr is within [base..base+size)
+            // means we can check if ptr - (base + size) < 0
+            if ptr < (base + size) {
+                return Some((base, val));
+            }
+        }
+        None
+    }
+
     pub fn find_alloc_precise(&self, ptr: P) -> Option<&T> {
         self.ptrs.get(&ptr)
     }
