@@ -69,7 +69,7 @@ brw_compile_tes(const struct brw_compiler *compiler,
    struct brw_tes_prog_data *prog_data = params->prog_data;
    const unsigned dispatch_width = brw_geometry_stage_dispatch_width(compiler->devinfo);
 
-   const bool debug_enabled = brw_should_print_shader(nir, DEBUG_TES);
+   const bool debug_enabled = brw_should_print_shader(nir, DEBUG_TES, params->base.source_hash);
 
    brw_prog_data_init(&prog_data->base.base, &params->base);
 
@@ -84,7 +84,7 @@ brw_compile_tes(const struct brw_compiler *compiler,
 
    brw_compute_vue_map(devinfo, &prog_data->base.vue_map,
                        nir->info.outputs_written,
-                       nir->info.separate_shader, 1);
+                       key->base.vue_layout, 1);
 
    unsigned output_size_bytes = prog_data->base.vue_map.num_slots * 4 * 4;
 
