@@ -143,8 +143,7 @@ brw_nir_ubo_surface_index_get_bti(nir_src src)
 /* Returns true if a fragment shader needs at least one render target */
 static inline bool
 brw_nir_fs_needs_null_rt(const struct intel_device_info *devinfo,
-                         nir_shader *nir,
-                         bool multisample_fbo, bool alpha_to_coverage)
+                         nir_shader *nir, bool alpha_to_coverage)
 {
    assert(nir->info.stage == MESA_SHADER_FRAGMENT);
 
@@ -158,15 +157,11 @@ brw_nir_fs_needs_null_rt(const struct intel_device_info *devinfo,
     * output.
     */
    if (nir->info.outputs_written & (BITFIELD_BIT(FRAG_RESULT_DEPTH) |
-                                    BITFIELD_BIT(FRAG_RESULT_STENCIL)))
+                                    BITFIELD_BIT(FRAG_RESULT_STENCIL) |
+                                    BITFIELD64_BIT(FRAG_RESULT_SAMPLE_MASK)))
       return true;
 
-   uint64_t relevant_outputs = 0;
-   if (multisample_fbo)
-      relevant_outputs |= BITFIELD64_BIT(FRAG_RESULT_SAMPLE_MASK);
-
-   return (alpha_to_coverage ||
-           (nir->info.outputs_written & relevant_outputs) != 0);
+   return alpha_to_coverage;
 }
 
 void brw_preprocess_nir(const struct brw_compiler *compiler,
@@ -180,9 +175,12 @@ brw_nir_link_shaders(const struct brw_compiler *compiler,
 bool brw_nir_lower_cs_intrinsics(nir_shader *nir,
                                  const struct intel_device_info *devinfo,
                                  struct brw_cs_prog_data *prog_data);
-bool brw_nir_lower_alpha_to_coverage(nir_shader *shader,
-                                     const struct brw_wm_prog_key *key,
-                                     const struct brw_wm_prog_data *prog_data);
+bool brw_nir_lower_alpha_to_coverage(nir_shader *shader);
+bool brw_needs_vertex_attributes_bypass(const nir_shader *shader);
+void brw_nir_lower_fs_barycentrics(nir_shader *shader);
+bool brw_nir_lower_fs_msaa(nir_shader *shader,
+                           const struct brw_wm_prog_key *key);
+
 void brw_nir_lower_vs_inputs(nir_shader *nir);
 void brw_nir_lower_vue_inputs(nir_shader *nir,
                               const struct intel_vue_map *vue_map);
@@ -198,14 +196,14 @@ void brw_nir_lower_fs_outputs(nir_shader *nir);
 bool brw_nir_lower_cmat(nir_shader *nir, unsigned subgroup_size);
 
 struct brw_nir_lower_storage_image_opts {
-   const struct intel_device_info *devinfo;
-
    bool lower_loads;
    bool lower_stores;
    bool lower_stores_64bit;
+   bool lower_loads_without_formats;
 };
 
 bool brw_nir_lower_storage_image(nir_shader *nir,
+                                 const struct brw_compiler *compiler,
                                  const struct brw_nir_lower_storage_image_opts *opts);
 
 bool brw_nir_lower_texel_address(nir_shader *shader,
@@ -217,6 +215,8 @@ bool brw_nir_lower_texture(nir_shader *nir,
 
 bool brw_nir_lower_sample_index_in_coord(nir_shader *nir);
 
+bool brw_nir_lower_immediate_offsets(nir_shader *shader);
+
 bool brw_nir_lower_mem_access_bit_sizes(nir_shader *shader,
                                         const struct
                                         intel_device_info *devinfo);
@@ -249,6 +249,7 @@ unsigned brw_nir_api_subgroup_size(const nir_shader *nir,
 
 enum brw_conditional_mod brw_cmod_for_nir_comparison(nir_op op);
 enum lsc_opcode lsc_op_for_nir_intrinsic(const nir_intrinsic_instr *intrin);
+enum brw_reg_type brw_type_for_base_type(enum glsl_base_type base_type);
 enum brw_reg_type brw_type_for_nir_type(const struct intel_device_info *devinfo,
                                         nir_alu_type type);
 
@@ -317,6 +318,52 @@ bool brw_nir_uses_inline_data(nir_shader *shader);
 nir_shader *
 brw_nir_from_spirv(void *mem_ctx, const uint32_t *spirv, size_t spirv_size);
 
+nir_variable *
+brw_nir_find_complete_variable_with_location(nir_shader *shader,
+                                             nir_variable_mode mode,
+                                             int location);
+
+nir_def *
+brw_nir_vertex_attribute_offset(nir_builder *b,
+                                nir_def *attr_idx,
+                                const struct intel_device_info *devinfo);
+
+static inline bool
+brw_nir_mesh_shader_needs_wa_18019110168(const struct intel_device_info *devinfo,
+                                         nir_shader *shader)
+{
+   return intel_needs_workaround(devinfo, 18019110168) &&
+      (shader->info.outputs_written & (VARYING_BIT_CLIP_DIST0 |
+                                       VARYING_BIT_CLIP_DIST1)) &&
+      (shader->info.per_primitive_outputs & ~(VARYING_BIT_PRIMITIVE_INDICES |
+                                              VARYING_BIT_PRIMITIVE_COUNT));
+}
+
+static inline bool
+brw_nir_fragment_shader_needs_wa_18019110168(const struct intel_device_info *devinfo,
+                                             enum intel_sometimes mesh_input,
+                                             nir_shader *shader)
+{
+   return intel_needs_workaround(devinfo, 18019110168) &&
+      mesh_input != INTEL_NEVER &&
+      (shader->info.per_primitive_inputs != 0 ||
+       (shader->info.inputs_read & VARYING_BIT_PRIMITIVE_ID));
+}
+
+void
+brw_nir_mesh_convert_attrs_prim_to_vert(struct nir_shader *nir,
+                                        struct brw_compile_mesh_params *params,
+                                        int *wa_mapping);
+
+bool
+brw_nir_frag_convert_attrs_prim_to_vert(struct nir_shader *nir,
+                                        const int *wa_mapping);
+
+bool
+brw_nir_frag_convert_attrs_prim_to_vert_indirect(struct nir_shader *nir,
+                                                 const struct intel_device_info *devinfo,
+                                                 struct brw_compile_fs_params *params);
+
 #ifdef __cplusplus
 }
 #endif
