@@ -23,7 +23,6 @@
 
 #include "anv_private.h"
 #include "drm-uapi/drm_fourcc.h"
-#include "vk_android.h"
 #include "vk_enum_defines.h"
 #include "vk_enum_to_str.h"
 #include "vk_format.h"
@@ -532,8 +531,7 @@ anv_get_format(const struct anv_physical_device *device, VkFormat vk_format)
 
    const struct anv_format *format =
       &anv_formats[ext_number].formats[enum_offset];
-   if (format->planes[0].isl_format == ISL_FORMAT_UNSUPPORTED &&
-       format->planes[0].vbo_format == ISL_FORMAT_UNSUPPORTED)
+   if (format->planes[0].isl_format == ISL_FORMAT_UNSUPPORTED)
       return NULL;
 
    /* This format is only available if custom border colors without format is
@@ -646,8 +644,7 @@ anv_get_image_format_features2(const struct anv_physical_device *physical_device
                             VK_IMAGE_CREATE_SPARSE_RESIDENCY_BIT |
                             VK_IMAGE_CREATE_SPARSE_ALIASED_BIT)) != 0;
 
-   if (anv_format == NULL ||
-       anv_format->planes[0].isl_format == ISL_FORMAT_UNSUPPORTED)
+   if (anv_format == NULL)
       return 0;
 
    assert((isl_mod_info != NULL) ==
@@ -661,19 +658,17 @@ anv_get_image_format_features2(const struct anv_physical_device *physical_device
    if (anv_is_compressed_format_emulated(physical_device, vk_format)) {
       assert(isl_format_is_compressed(anv_format->planes[0].isl_format));
 
-      /* Require optimal tiling so that we can decompress on upload */
-      if (vk_tiling == VK_IMAGE_TILING_OPTIMAL) {
-         /* Required features for compressed formats */
-         flags |= VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_BIT |
-                  VK_FORMAT_FEATURE_2_BLIT_SRC_BIT |
-                  VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_FILTER_LINEAR_BIT |
-                  VK_FORMAT_FEATURE_2_TRANSFER_SRC_BIT |
-                  VK_FORMAT_FEATURE_2_TRANSFER_DST_BIT;
-      } else if (vk_tiling == VK_IMAGE_TILING_LINEAR) {
-         /* Images used for transfers */
-         flags |= VK_FORMAT_FEATURE_2_TRANSFER_SRC_BIT |
-                  VK_FORMAT_FEATURE_2_TRANSFER_DST_BIT;
-      }
+      /* require optimal tiling so that we can decompress on upload */
+      if (vk_tiling != VK_IMAGE_TILING_OPTIMAL)
+         return 0;
+
+      /* required features for compressed formats */
+      flags |= VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_BIT |
+               VK_FORMAT_FEATURE_2_BLIT_SRC_BIT |
+               VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_FILTER_LINEAR_BIT |
+               VK_FORMAT_FEATURE_2_TRANSFER_SRC_BIT |
+               VK_FORMAT_FEATURE_2_TRANSFER_DST_BIT;
+
       return flags;
    }
 
@@ -773,9 +768,15 @@ anv_get_image_format_features2(const struct anv_physical_device *physical_device
 
    /* Load/store is determined based on base format.  This prevents RGB
     * formats from showing up as load/store capable.
+    *
+    * Typed writes match with storage write without format. For storage read
+    * without format, either HW can do it (isl_format_supports_typed_reads) or
+    * do in-shader conversion for isl_is_storage_image_format format.
     */
    if ((anv_format->flags & ANV_FORMAT_FLAG_STORAGE_FORMAT_EMULATED) == 0) {
-      if (isl_format_supports_typed_reads(devinfo, base_isl_format))
+      if (isl_format_supports_typed_reads(devinfo, base_isl_format) ||
+          (physical_device->instance->emulate_read_without_format &&
+           isl_is_storage_image_format(devinfo, plane_format.isl_format)))
          flags |= VK_FORMAT_FEATURE_2_STORAGE_READ_WITHOUT_FORMAT_BIT;
       if (isl_format_supports_typed_writes(devinfo, base_isl_format))
          flags |= VK_FORMAT_FEATURE_2_STORAGE_WRITE_WITHOUT_FORMAT_BIT;
@@ -1083,29 +1084,8 @@ get_buffer_format_features2(const struct anv_physical_device *physical_device,
       }
 
       if (devinfo->has_ray_tracing) {
-#if ANV_SUPPORT_RT_GRL
-         switch (vk_format) {
-         case VK_FORMAT_R32G32_SFLOAT:
-         case VK_FORMAT_R32G32B32_SFLOAT:
-         case VK_FORMAT_R16G16_SFLOAT:
-         case VK_FORMAT_R16G16B16A16_SFLOAT:
-         case VK_FORMAT_R16G16_SNORM:
-         case VK_FORMAT_R16G16B16A16_SNORM:
-         case VK_FORMAT_R16G16B16A16_UNORM:
-         case VK_FORMAT_R16G16_UNORM:
-         case VK_FORMAT_R8G8B8A8_UNORM:
-         case VK_FORMAT_R8G8_UNORM:
-         case VK_FORMAT_R8G8B8A8_SNORM:
-         case VK_FORMAT_R8G8_SNORM:
-            flags |= VK_FORMAT_FEATURE_ACCELERATION_STRUCTURE_VERTEX_BUFFER_BIT_KHR;
-            break;
-         default:
-            break;
-         }
-#else
          if (vk_acceleration_struct_vtx_format_supported(vk_format))
             flags |= VK_FORMAT_FEATURE_ACCELERATION_STRUCTURE_VERTEX_BUFFER_BIT_KHR;
-#endif
       }
    }
 
