@@ -213,8 +213,7 @@ anv_shader_bin_create(struct anv_device *device,
                       const struct brw_compile_stats *stats, uint32_t num_stats,
                       const nir_xfb_info *xfb_info_in,
                       const struct anv_pipeline_bind_map *bind_map,
-                      const struct anv_push_descriptor_info *push_desc_info,
-                      enum anv_dynamic_push_bits dynamic_push_values)
+                      const struct anv_push_descriptor_info *push_desc_info)
 {
    VK_MULTIALLOC(ma);
    VK_MULTIALLOC_DECL(&ma, struct anv_shader_bin, shader, 1);
@@ -236,8 +235,6 @@ anv_shader_bin_create(struct anv_device *device,
    VK_MULTIALLOC_DECL(&ma, struct anv_pipeline_embedded_sampler_binding,
                       embedded_sampler_to_binding,
                       bind_map->embedded_sampler_count);
-   VK_MULTIALLOC_DECL(&ma, struct brw_kernel_arg_desc, kernel_args,
-                      bind_map->kernel_arg_count);
    VK_MULTIALLOC_DECL(&ma, struct anv_embedded_sampler *, embedded_samplers,
                       bind_map->embedded_sampler_count);
 
@@ -250,6 +247,8 @@ anv_shader_bin_create(struct anv_device *device,
                                  &anv_shader_bin_ops, obj_key_data, key_size);
 
    shader->stage = stage;
+   if(INTEL_DEBUG(DEBUG_SHOW_SHADER_STAGE))
+      fprintf(stderr, "Stage: %s\n", gl_shader_stage_name(shader->stage));
 
    shader->kernel =
       anv_state_pool_alloc(&device->instruction_state_pool, kernel_size, 64);
@@ -273,7 +272,12 @@ anv_shader_bin_create(struct anv_device *device,
       prog_data_in->const_data_offset;
 
    int rv_count = 0;
-   struct brw_shader_reloc_value reloc_values[10];
+   struct brw_shader_reloc_value reloc_values[11];
+   assert((device->physical->va.instruction_state_pool.addr & 0xffffffff) == 0);
+   reloc_values[rv_count++] = (struct brw_shader_reloc_value) {
+      .id = BRW_SHADER_RELOC_INSTRUCTION_BASE_ADDR_HIGH,
+      .value = device->physical->va.instruction_state_pool.addr >> 32,
+   };
    assert((device->physical->va.dynamic_visible_pool.addr & 0xffffffff) == 0);
    reloc_values[rv_count++] = (struct brw_shader_reloc_value) {
       .id = BRW_SHADER_RELOC_DESCRIPTORS_BUFFER_ADDR_HIGH,
@@ -365,8 +369,6 @@ anv_shader_bin_create(struct anv_device *device,
       shader->xfb_info = NULL;
    }
 
-   shader->dynamic_push_values = dynamic_push_values;
-
    typed_memcpy(&shader->push_desc_info, push_desc_info, 1);
 
    shader->bind_map = *bind_map;
@@ -387,10 +389,6 @@ anv_shader_bin_create(struct anv_device *device,
                 bind_map->input_attachments,
                 ARRAY_SIZE(bind_map->input_attachments));
 
-   typed_memcpy(kernel_args, bind_map->kernel_args,
-                bind_map->kernel_arg_count);
-   shader->bind_map.kernel_args = kernel_args;
-
    return shader;
 }
 
@@ -432,8 +430,6 @@ anv_shader_bin_serialize(struct vk_pipeline_cache_object *object,
       blob_write_uint32(blob, 0);
    }
 
-   blob_write_uint32(blob, shader->dynamic_push_values);
-
    blob_write_uint32(blob, shader->push_desc_info.used_descriptors);
    blob_write_uint32(blob, shader->push_desc_info.fully_promoted_ubo_descriptors);
    blob_write_uint8(blob, shader->push_desc_info.used_set_buffer);
@@ -447,11 +443,6 @@ anv_shader_bin_serialize(struct vk_pipeline_cache_object *object,
    blob_write_uint32(blob, shader->bind_map.surface_count);
    blob_write_uint32(blob, shader->bind_map.sampler_count);
    blob_write_uint32(blob, shader->bind_map.embedded_sampler_count);
-   if (shader->stage == MESA_SHADER_KERNEL) {
-      uint32_t packed = (uint32_t)shader->bind_map.kernel_args_size << 16 |
-                        (uint32_t)shader->bind_map.kernel_arg_count;
-      blob_write_uint32(blob, packed);
-   }
    blob_write_bytes(blob, shader->bind_map.surface_to_descriptor,
                     shader->bind_map.surface_count *
                     sizeof(*shader->bind_map.surface_to_descriptor));
@@ -463,9 +454,6 @@ anv_shader_bin_serialize(struct vk_pipeline_cache_object *object,
                     sizeof(*shader->bind_map.embedded_sampler_to_binding));
    blob_write_bytes(blob, shader->bind_map.input_attachments,
                     sizeof(shader->bind_map.input_attachments));
-   blob_write_bytes(blob, shader->bind_map.kernel_args,
-                    shader->bind_map.kernel_arg_count *
-                    sizeof(*shader->bind_map.kernel_args));
    blob_write_bytes(blob, shader->bind_map.push_ranges,
                     sizeof(shader->bind_map.push_ranges));
 
@@ -507,8 +495,6 @@ anv_shader_bin_deserialize(struct vk_pipeline_cache *cache,
    if (xfb_size)
       xfb_info = blob_read_bytes(blob, xfb_size);
 
-   enum anv_dynamic_push_bits dynamic_push_values = blob_read_uint32(blob);
-
    struct anv_push_descriptor_info push_desc_info = {};
    push_desc_info.used_descriptors = blob_read_uint32(blob);
    push_desc_info.fully_promoted_ubo_descriptors = blob_read_uint32(blob);
@@ -521,11 +507,6 @@ anv_shader_bin_deserialize(struct vk_pipeline_cache *cache,
    bind_map.surface_count = blob_read_uint32(blob);
    bind_map.sampler_count = blob_read_uint32(blob);
    bind_map.embedded_sampler_count = blob_read_uint32(blob);
-   if (stage == MESA_SHADER_KERNEL) {
-      uint32_t packed = blob_read_uint32(blob);
-      bind_map.kernel_args_size = (uint16_t)(packed >> 16);
-      bind_map.kernel_arg_count = (uint16_t)packed;
-   }
    bind_map.surface_to_descriptor = (void *)
       blob_read_bytes(blob, bind_map.surface_count *
                             sizeof(*bind_map.surface_to_descriptor));
@@ -537,9 +518,6 @@ anv_shader_bin_deserialize(struct vk_pipeline_cache *cache,
                             sizeof(*bind_map.embedded_sampler_to_binding));
    blob_copy_bytes(blob, bind_map.input_attachments,
                    sizeof(bind_map.input_attachments));
-   bind_map.kernel_args = (void *)
-      blob_read_bytes(blob, bind_map.kernel_arg_count *
-                            sizeof(*bind_map.kernel_args));
    blob_copy_bytes(blob, bind_map.push_ranges, sizeof(bind_map.push_ranges));
 
    if (blob->overrun) {
@@ -553,8 +531,7 @@ anv_shader_bin_deserialize(struct vk_pipeline_cache *cache,
                             kernel_data, kernel_size,
                             &prog_data.base, prog_data_size,
                             stats, num_stats, xfb_info, &bind_map,
-                            &push_desc_info,
-                            dynamic_push_values);
+                            &push_desc_info);
 
    ralloc_free(mem_ctx);
 
@@ -611,8 +588,7 @@ anv_device_upload_kernel(struct anv_device *device,
                             params->num_stats,
                             params->xfb_info,
                             params->bind_map,
-                            params->push_desc_info,
-                            params->dynamic_push_values);
+                            params->push_desc_info);
    if (shader == NULL)
       return NULL;
 
@@ -677,7 +653,6 @@ anv_load_fp64_shader(struct anv_device *device)
       .Int8 = true,
       .Int16 = true,
       .Int64 = true,
-      .Shader = true,
    };
 
    struct spirv_to_nir_options spirv_options = {
@@ -695,9 +670,9 @@ anv_load_fp64_shader(struct anv_device *device)
 
    nir_validate_shader(nir, "after spirv_to_nir");
 
-   NIR_PASS_V(nir, nir_lower_variable_initializers, nir_var_function_temp);
-   NIR_PASS_V(nir, nir_lower_returns);
-   NIR_PASS_V(nir, nir_inline_functions);
+   NIR_PASS(_, nir, nir_lower_variable_initializers, nir_var_function_temp);
+   NIR_PASS(_, nir, nir_lower_returns);
+   NIR_PASS(_, nir, nir_inline_functions);
 
    anv_device_upload_nir(device, device->internal_cache,
                          nir, sha1);
