@@ -28,6 +28,7 @@
 #include "genxml/genX_rt_pack.h"
 
 #include "common/intel_compute_slm.h"
+#include "common/intel_common.h"
 #include "common/intel_genX_state_brw.h"
 #include "common/intel_l3_config.h"
 #include "common/intel_sample_positions.h"
@@ -213,8 +214,10 @@ emit_ves_vf_instancing(struct anv_batch *batch,
    }
 
    u_foreach_bit(a, vi->attributes_valid) {
-      enum isl_format format = anv_get_vbo_format(
-         device->physical, vi->attributes[a].format);
+      enum isl_format format = anv_get_vbo_format(device->physical,
+                                                  vi->attributes[a].format,
+                                                  VK_IMAGE_ASPECT_COLOR_BIT,
+                                                  VK_IMAGE_TILING_LINEAR);
       assume(format < ISL_NUM_FORMATS);
 
       uint32_t binding = vi->attributes[a].binding;
@@ -529,7 +532,7 @@ emit_urb_setup_mesh(struct anv_graphics_pipeline *pipeline,
    const struct intel_mesh_urb_allocation alloc =
       intel_get_mesh_urb_config(devinfo, pipeline->base.base.l3_config,
                                 task_prog_data ? task_prog_data->map.size_dw : 0,
-                                mesh_prog_data->map.size_dw);
+                                mesh_prog_data->map.size / 4);
 
    /* Zero out the primitive pipeline URB allocations. */
    for (int i = 0; i <= MESA_SHADER_GEOMETRY; i++) {
@@ -639,76 +642,81 @@ sbe_primitive_id_override(struct anv_graphics_pipeline *pipeline)
          get_mesh_prog_data(pipeline);
       const struct brw_mue_map *mue = &mesh_prog_data->map;
       return (wm_prog_data->inputs & VARYING_BIT_PRIMITIVE_ID) &&
-              mue->start_dw[VARYING_SLOT_PRIMITIVE_ID] == -1;
+              mue->per_primitive_offsets[VARYING_SLOT_PRIMITIVE_ID] == -1;
    }
 
    const struct intel_vue_map *fs_input_map =
       &anv_pipeline_get_last_vue_prog_data(pipeline)->vue_map;
 
    return (wm_prog_data->inputs & VARYING_BIT_PRIMITIVE_ID) &&
-          fs_input_map->varying_to_slot[VARYING_SLOT_PRIMITIVE_ID] == -1;
+          (fs_input_map->slots_valid & VARYING_BIT_PRIMITIVE_ID) == 0;
 }
 
 static void
 emit_3dstate_sbe(struct anv_graphics_pipeline *pipeline)
 {
    const struct brw_wm_prog_data *wm_prog_data = get_wm_prog_data(pipeline);
+   const struct brw_mesh_prog_data *mesh_prog_data =
+      get_mesh_prog_data(pipeline);
+   UNUSED const struct anv_device *device = pipeline->base.base.device;
 
    if (!anv_pipeline_has_stage(pipeline, MESA_SHADER_FRAGMENT)) {
       anv_pipeline_emit(pipeline, final.sbe, GENX(3DSTATE_SBE), sbe);
       anv_pipeline_emit(pipeline, final.sbe_swiz, GENX(3DSTATE_SBE_SWIZ), sbe);
 #if GFX_VERx10 >= 125
-      if (anv_pipeline_is_mesh(pipeline))
+      if (device->vk.enabled_extensions.EXT_mesh_shader)
          anv_pipeline_emit(pipeline, final.sbe_mesh, GENX(3DSTATE_SBE_MESH), sbe);
 #endif
       return;
    }
 
+   const struct intel_vue_map *vue_map =
+      anv_pipeline_is_mesh(pipeline) ?
+      &get_mesh_prog_data(pipeline)->map.vue_map :
+      &anv_pipeline_get_last_vue_prog_data(pipeline)->vue_map;
+
    anv_pipeline_emit(pipeline, final.sbe, GENX(3DSTATE_SBE), sbe) {
    anv_pipeline_emit(pipeline, final.sbe_swiz, GENX(3DSTATE_SBE_SWIZ), swiz) {
+      int max_source_attr = 0;
+      uint32_t vertex_read_offset, vertex_read_length, vertex_varyings, flat_inputs;
+      brw_compute_sbe_per_vertex_urb_read(
+         vue_map, mesh_prog_data != NULL,
+         mesh_prog_data ? mesh_prog_data->map.wa_18019110168_active : false,
+         wm_prog_data,
+         &vertex_read_offset, &vertex_read_length, &vertex_varyings,
+         &pipeline->primitive_id_index,
+         &flat_inputs);
+
+      pipeline->first_vue_slot = vertex_read_offset * 2;
 
-      /* TODO(mesh): Figure out cases where we need attribute swizzling.  See also
-       * calculate_urb_setup() and related functions.
-       */
       sbe.AttributeSwizzleEnable = anv_pipeline_is_primitive(pipeline);
       sbe.PointSpriteTextureCoordinateOrigin = UPPERLEFT;
-      sbe.NumberofSFOutputAttributes = wm_prog_data->num_varying_inputs;
-      sbe.ConstantInterpolationEnable = wm_prog_data->flat_inputs;
+      sbe.ConstantInterpolationEnable = flat_inputs;
+      sbe.NumberofSFOutputAttributes = vertex_varyings;
+#if GFX_VERx10 >= 200
+      sbe.VertexAttributesBypass = wm_prog_data->vertex_attributes_bypass;
+#endif
 
       for (unsigned i = 0; i < 32; i++)
          sbe.AttributeActiveComponentFormat[i] = ACF_XYZW;
 
+      /* As far as we can test, some of the fields in 3DSTATE_SBE & all of
+       * 3DSTATE_SBE_SWIZ has no effect when the pipeline is using Mesh so
+       * don't bother filling those fields.
+       */
       if (anv_pipeline_is_primitive(pipeline)) {
-         const struct intel_vue_map *fs_input_map =
-            &anv_pipeline_get_last_vue_prog_data(pipeline)->vue_map;
-
-         int first_slot =
-            brw_compute_first_urb_slot_required(wm_prog_data->inputs,
-                                                fs_input_map);
-         assert(first_slot % 2 == 0);
-         unsigned urb_entry_read_offset = first_slot / 2;
-         int max_source_attr = 0;
          for (uint8_t idx = 0; idx < wm_prog_data->urb_setup_attribs_count; idx++) {
             uint8_t attr = wm_prog_data->urb_setup_attribs[idx];
             int input_index = wm_prog_data->urb_setup[attr];
 
             assert(0 <= input_index);
 
-            /* gl_Viewport, gl_Layer and FragmentShadingRateKHR are stored in the
-             * VUE header
-             */
-            if (attr == VARYING_SLOT_VIEWPORT ||
-                attr == VARYING_SLOT_LAYER ||
-                attr == VARYING_SLOT_PRIMITIVE_SHADING_RATE) {
-               continue;
-            }
-
             if (attr == VARYING_SLOT_PNTC) {
                sbe.PointSpriteTextureCoordinateEnable = 1 << input_index;
                continue;
             }
 
-            const int slot = fs_input_map->varying_to_slot[attr];
+            const int slot = vue_map->varying_to_slot[attr];
 
             if (slot == -1) {
                /* This attribute does not exist in the VUE--that means that
@@ -729,7 +737,7 @@ emit_3dstate_sbe(struct anv_graphics_pipeline *pipeline)
             /* We have to subtract two slots to account for the URB entry
              * output read offset in the VS and GS stages.
              */
-            const int source_attr = slot - 2 * urb_entry_read_offset;
+            const int source_attr = slot - 2 * vertex_read_offset;
             assert(source_attr >= 0 && source_attr < 32);
             max_source_attr = MAX2(max_source_attr, source_attr);
             /* The hardware can only do overrides on 16 overrides at a time,
@@ -743,77 +751,46 @@ emit_3dstate_sbe(struct anv_graphics_pipeline *pipeline)
                assert(source_attr == input_index);
          }
 
-         sbe.VertexURBEntryReadOffset = urb_entry_read_offset;
-         sbe.VertexURBEntryReadLength = DIV_ROUND_UP(max_source_attr + 1, 2);
+         sbe.VertexURBEntryReadOffset = vertex_read_offset;
+         sbe.VertexURBEntryReadLength = vertex_read_length;
          sbe.ForceVertexURBEntryReadOffset = true;
          sbe.ForceVertexURBEntryReadLength = true;
+      }
+
+      /* Ask the hardware to supply PrimitiveID if the fragment shader reads
+       * it but a previous stage didn't write one.
+       */
+      if (sbe_primitive_id_override(pipeline)) {
+         sbe.PrimitiveIDOverrideAttributeSelect =
+            wm_prog_data->urb_setup[VARYING_SLOT_PRIMITIVE_ID];
+         sbe.PrimitiveIDOverrideComponentX = true;
+         sbe.PrimitiveIDOverrideComponentY = true;
+         sbe.PrimitiveIDOverrideComponentZ = true;
+         sbe.PrimitiveIDOverrideComponentW = true;
+      }
 
-         /* Ask the hardware to supply PrimitiveID if the fragment shader
-          * reads it but a previous stage didn't write one.
-          */
-         if (sbe_primitive_id_override(pipeline)) {
-            sbe.PrimitiveIDOverrideAttributeSelect =
-               wm_prog_data->urb_setup[VARYING_SLOT_PRIMITIVE_ID];
-            sbe.PrimitiveIDOverrideComponentX = true;
-            sbe.PrimitiveIDOverrideComponentY = true;
-            sbe.PrimitiveIDOverrideComponentZ = true;
-            sbe.PrimitiveIDOverrideComponentW = true;
-         }
-      } else {
-         assert(anv_pipeline_is_mesh(pipeline));
 #if GFX_VERx10 >= 125
-         const struct brw_mesh_prog_data *mesh_prog_data = get_mesh_prog_data(pipeline);
+      if (device->vk.enabled_extensions.EXT_mesh_shader) {
          anv_pipeline_emit(pipeline, final.sbe_mesh,
                            GENX(3DSTATE_SBE_MESH), sbe_mesh) {
-            const struct brw_mue_map *mue = &mesh_prog_data->map;
+            if (mesh_prog_data == NULL)
+               continue;
 
-            assert(mue->per_vertex_header_size_dw % 8 == 0);
-            sbe_mesh.PerVertexURBEntryOutputReadOffset = mue->per_vertex_header_size_dw / 8;
-            sbe_mesh.PerVertexURBEntryOutputReadLength = DIV_ROUND_UP(mue->per_vertex_data_size_dw, 8);
+            sbe_mesh.PerVertexURBEntryOutputReadOffset = vertex_read_offset;
+            sbe_mesh.PerVertexURBEntryOutputReadLength = vertex_read_length;
 
-            /* Clip distance array is passed in the per-vertex header so that
-             * it can be consumed by the HW. If user wants to read it in the
-             * FS, adjust the offset and length to cover it. Conveniently it
-             * is at the end of the per-vertex header, right before per-vertex
-             * attributes.
-             *
-             * Note that FS attribute reading must be aware that the clip
-             * distances have fixed position.
-             */
-            if (mue->per_vertex_header_size_dw > 8 &&
-                (wm_prog_data->urb_setup[VARYING_SLOT_CLIP_DIST0] >= 0 ||
-                 wm_prog_data->urb_setup[VARYING_SLOT_CLIP_DIST1] >= 0)) {
-               sbe_mesh.PerVertexURBEntryOutputReadOffset -= 1;
-               sbe_mesh.PerVertexURBEntryOutputReadLength += 1;
-            }
+            uint32_t prim_read_offset, prim_read_length;
+            brw_compute_sbe_per_primitive_urb_read(wm_prog_data->per_primitive_inputs,
+                                                   wm_prog_data->num_per_primitive_inputs,
+                                                   &mesh_prog_data->map,
+                                                   &prim_read_offset,
+                                                   &prim_read_length);
 
-            if (mue->user_data_in_vertex_header) {
-               sbe_mesh.PerVertexURBEntryOutputReadOffset -= 1;
-               sbe_mesh.PerVertexURBEntryOutputReadLength += 1;
-            }
-
-            assert(mue->per_primitive_header_size_dw % 8 == 0);
-            sbe_mesh.PerPrimitiveURBEntryOutputReadOffset =
-               mue->per_primitive_header_size_dw / 8;
-            sbe_mesh.PerPrimitiveURBEntryOutputReadLength =
-               DIV_ROUND_UP(mue->per_primitive_data_size_dw, 8);
-
-            /* Just like with clip distances, if Primitive Shading Rate,
-             * Viewport Index or Layer is read back in the FS, adjust the
-             * offset and length to cover the Primitive Header, where PSR,
-             * Viewport Index & Layer are stored.
-             */
-            if (wm_prog_data->urb_setup[VARYING_SLOT_VIEWPORT] >= 0 ||
-                wm_prog_data->urb_setup[VARYING_SLOT_PRIMITIVE_SHADING_RATE] >= 0 ||
-                wm_prog_data->urb_setup[VARYING_SLOT_LAYER] >= 0 ||
-                mue->user_data_in_primitive_header) {
-               assert(sbe_mesh.PerPrimitiveURBEntryOutputReadOffset > 0);
-               sbe_mesh.PerPrimitiveURBEntryOutputReadOffset -= 1;
-               sbe_mesh.PerPrimitiveURBEntryOutputReadLength += 1;
-            }
+            sbe_mesh.PerPrimitiveURBEntryOutputReadOffset = prim_read_offset;
+            sbe_mesh.PerPrimitiveURBEntryOutputReadLength = prim_read_length;
          }
-#endif
       }
+#endif
    }
    }
 }
@@ -844,7 +821,7 @@ emit_rs_state(struct anv_graphics_pipeline *pipeline,
       } else {
          assert(anv_pipeline_is_mesh(pipeline));
          const struct brw_mesh_prog_data *mesh_prog_data = get_mesh_prog_data(pipeline);
-         point_from_shader = mesh_prog_data->map.start_dw[VARYING_SLOT_PSIZ] >= 0;
+         point_from_shader = mesh_prog_data->map.vue_map.slots_valid & VARYING_BIT_PSIZ;
       }
 
       if (point_from_shader) {
@@ -895,7 +872,7 @@ emit_3dstate_clip(struct anv_graphics_pipeline *pipeline,
          const struct brw_mesh_prog_data *mesh_prog_data = get_mesh_prog_data(pipeline);
 
          clip.ForceZeroRTAIndexEnable =
-            mesh_prog_data->map.start_dw[VARYING_SLOT_LAYER] < 0;
+            mesh_prog_data->map.per_primitive_offsets[VARYING_SLOT_LAYER] < 0;
       }
 
       clip.NonPerspectiveBarycentricEnable = wm_prog_data ?
@@ -903,11 +880,15 @@ emit_3dstate_clip(struct anv_graphics_pipeline *pipeline,
    }
 
 #if GFX_VERx10 >= 125
-   if (anv_pipeline_is_mesh(pipeline)) {
-      const struct brw_mesh_prog_data *mesh_prog_data = get_mesh_prog_data(pipeline);
+   const struct anv_device *device = pipeline->base.base.device;
+   if (device->vk.enabled_extensions.EXT_mesh_shader) {
       anv_pipeline_emit(pipeline, final.clip_mesh,
                         GENX(3DSTATE_CLIP_MESH), clip_mesh) {
-         clip_mesh.PrimitiveHeaderEnable = mesh_prog_data->map.per_primitive_header_size_dw > 0;
+         if (!anv_pipeline_is_mesh(pipeline))
+            continue;
+
+         const struct brw_mesh_prog_data *mesh_prog_data = get_mesh_prog_data(pipeline);
+         clip_mesh.PrimitiveHeaderEnable = mesh_prog_data->map.has_per_primitive_header;
          clip_mesh.UserClipDistanceClipTestEnableBitmask = mesh_prog_data->clip_distance_mask;
          clip_mesh.UserClipDistanceCullTestEnableBitmask = mesh_prog_data->cull_distance_mask;
       }
@@ -1596,7 +1577,6 @@ emit_3dstate_ps(struct anv_graphics_pipeline *pipeline,
       ps.BindingTableEntryCount     = fs_bin->bind_map.surface_count;
 #if GFX_VER < 20
       ps.PushConstantEnable         =
-         devinfo->needs_null_push_constant_tbimr_workaround ||
          wm_prog_data->base.nr_params > 0 ||
          wm_prog_data->base.ubo_ranges[0].length;
 #endif
@@ -1903,9 +1883,9 @@ emit_mesh_state(struct anv_graphics_pipeline *pipeline)
 
       mesh.MaximumPrimitiveCount             = MAX2(mesh_prog_data->map.max_primitives, 1) - 1;
       mesh.OutputTopology                    = output_topology;
-      mesh.PerVertexDataPitch                = mesh_prog_data->map.per_vertex_pitch_dw / 8;
-      mesh.PerPrimitiveDataPresent           = mesh_prog_data->map.per_primitive_pitch_dw > 0;
-      mesh.PerPrimitiveDataPitch             = mesh_prog_data->map.per_primitive_pitch_dw / 8;
+      mesh.PerVertexDataPitch                = mesh_prog_data->map.per_vertex_stride / 32;
+      mesh.PerPrimitiveDataPresent           = mesh_prog_data->map.per_primitive_stride > 0;
+      mesh.PerPrimitiveDataPitch             = mesh_prog_data->map.per_primitive_stride / 32;
       mesh.IndexFormat                       = index_format;
 
       mesh.NumberofBarriers                  = mesh_prog_data->base.uses_barrier;
@@ -2012,10 +1992,6 @@ genX(graphics_pipeline_emit)(struct anv_graphics_pipeline *pipeline,
                            GENX(3DSTATE_MESH_SHADER), zero);
          anv_pipeline_emit(pipeline, final.mesh_distrib,
                            GENX(3DSTATE_MESH_DISTRIB), zero);
-         anv_pipeline_emit(pipeline, final.clip_mesh,
-                           GENX(3DSTATE_CLIP_MESH), zero);
-         anv_pipeline_emit(pipeline, final.sbe_mesh,
-                           GENX(3DSTATE_SBE_MESH), zero);
          anv_pipeline_emit(pipeline, final.task_control,
                            GENX(3DSTATE_TASK_CONTROL), zero);
          anv_pipeline_emit(pipeline, final.task_control_protected,
@@ -2105,6 +2081,8 @@ genX(compute_pipeline_emit)(struct anv_compute_pipeline *pipeline)
             .BindingTableEntryCount            = devinfo->verx10 == 125 ?
             0 : 1 + MIN2(shader->bind_map.surface_count, 30),
             .NumberofThreadsinGPGPUThreadGroup = dispatch.threads,
+            .ThreadGroupDispatchSize =
+               intel_compute_threads_group_dispatch_size(dispatch.threads),
             .SharedLocalMemorySize             =
             intel_compute_slm_encode_size(GFX_VER, prog_data->base.total_shared),
             .PreferredSLMAllocationSize        =
