@@ -69,14 +69,6 @@ choose_renderbuffer_format(struct gl_context *ctx,
 static void
 delete_renderbuffer(struct gl_context *ctx, struct gl_renderbuffer *rb)
 {
-   if (ctx) {
-      pipe_surface_release(ctx->pipe, &rb->surface_srgb);
-      pipe_surface_release(ctx->pipe, &rb->surface_linear);
-   } else {
-      pipe_surface_release_no_context(&rb->surface_srgb);
-      pipe_surface_release_no_context(&rb->surface_linear);
-   }
-   rb->surface = NULL;
    pipe_resource_reference(&rb->texture, NULL);
    free(rb->data);
    free(rb->Label);
@@ -149,11 +141,8 @@ renderbuffer_alloc_storage(struct gl_context * ctx,
                                            width, height);
    }
 
-   /* Free the old surface and texture
+   /* Free the old texture
     */
-   pipe_surface_reference(&rb->surface_srgb, NULL);
-   pipe_surface_reference(&rb->surface_linear, NULL);
-   rb->surface = NULL;
    pipe_resource_reference(&rb->texture, NULL);
 
    /* If an sRGB framebuffer is unsupported, sRGB formats behave like linear
@@ -196,7 +185,7 @@ renderbuffer_alloc_storage(struct gl_context * ctx,
              rb->_BaseFormat == GL_STENCIL_INDEX) {
             /* Find a supported depth-stencil format. */
             for (unsigned samples = start;
-                 samples <= ctx->Const.MaxDepthStencilFramebufferSamples;
+                 samples <= MAX_SAMPLES;
                  samples++) {
                format = choose_renderbuffer_format(ctx, internalFormat,
                                                    samples, samples);
@@ -210,7 +199,7 @@ renderbuffer_alloc_storage(struct gl_context * ctx,
          } else {
             /* Find a supported color format, samples >= storage_samples. */
             for (unsigned storage_samples = start_storage;
-                 storage_samples <= ctx->Const.MaxColorFramebufferStorageSamples;
+                 storage_samples <= MAX_SAMPLES;
                  storage_samples++) {
                for (unsigned samples = MAX2(start, storage_samples);
                     samples <= ctx->Const.MaxColorFramebufferSamples;
@@ -229,7 +218,7 @@ renderbuffer_alloc_storage(struct gl_context * ctx,
             found:;
          }
       } else {
-         for (unsigned samples = start; samples <= ctx->Const.MaxSamples;
+         for (unsigned samples = start; samples <= MAX_SAMPLES;
               samples++) {
             format = choose_renderbuffer_format(ctx, internalFormat,
                                                 samples, samples);
@@ -290,7 +279,7 @@ renderbuffer_alloc_storage(struct gl_context * ctx,
       return false;
 
    _mesa_update_renderbuffer_surface(ctx, rb);
-   return rb->surface != NULL;
+   return GL_TRUE;
 }
 
 /**
@@ -491,11 +480,12 @@ _mesa_map_renderbuffer(struct gl_context *ctx,
    else
       y2 = y;
 
-    map = pipe_texture_map(pipe,
-                            rb->texture,
-                            rb->surface->u.tex.level,
-                            rb->surface->u.tex.first_layer,
-                            transfer_flags, x, y2, w, h, &rb->transfer);
+   _mesa_update_renderbuffer_surface(ctx, rb);
+   map = pipe_texture_map(pipe,
+                           rb->texture,
+                           rb->surface.level,
+                           rb->surface.first_layer,
+                           transfer_flags, x, y2, w, h, &rb->transfer);
    if (map) {
       if (invert) {
          *rowStrideOut = -(int) rb->transfer->stride;
@@ -527,31 +517,34 @@ _mesa_unmap_renderbuffer(struct gl_context *ctx,
    rb->transfer = NULL;
 }
 
-void
-_mesa_regen_renderbuffer_surface(struct gl_context *ctx,
-                                 struct gl_renderbuffer *rb)
+enum pipe_format
+_mesa_renderbuffer_get_format(struct gl_context *ctx, struct gl_renderbuffer *rb)
 {
-   struct pipe_context *pipe = ctx->pipe;
-   struct pipe_resource *resource = rb->texture;
+   /*
+    * For winsys fbo, it is possible that the renderbuffer is sRGB-capable but
+    * the format of rb->texture is linear (because we have no control over
+    * the format).  Check rb->Format instead of rb->texture->format
+    * to determine if the rb is sRGB-capable.
+    */
+   bool enable_srgb = ctx->Color.sRGBEnabled && _mesa_is_format_srgb(rb->Format);
+   assert(rb->texture);
+   enum pipe_format format = rb->texture->format;
 
-   struct pipe_surface **psurf =
-      rb->surface_srgb ? &rb->surface_srgb : &rb->surface_linear;
-   struct pipe_surface *surf = *psurf;
-   /* create a new pipe_surface */
-   struct pipe_surface surf_tmpl;
-   memset(&surf_tmpl, 0, sizeof(surf_tmpl));
-   surf_tmpl.format = surf->format;
-   surf_tmpl.nr_samples = rb->rtt_nr_samples;
-   surf_tmpl.u.tex.level = surf->u.tex.level;
-   surf_tmpl.u.tex.first_layer = surf->u.tex.first_layer;
-   surf_tmpl.u.tex.last_layer = surf->u.tex.last_layer;
+   if (rb->is_rtt) {
+      const struct gl_texture_object *stTexObj = rb->TexImage->TexObject;
+      if (stTexObj->surface_based)
+         format = stTexObj->surface_format;
+   }
 
-   /* create -> destroy to avoid blowing up cached surfaces */
-   surf = pipe->create_surface(pipe, resource, &surf_tmpl);
-   pipe_surface_release(pipe, psurf);
-   *psurf = surf;
+   if (util_format_is_depth_or_stencil(format)) {
+      rb->format_srgb = format;
+      rb->format_linear = format;
+   } else {
+      rb->format_srgb = util_format_srgb(format);
+      rb->format_linear = util_format_linear(format);
+   }
 
-   rb->surface = *psurf;
+   return enable_srgb ? rb->format_srgb : rb->format_linear;
 }
 
 /**
@@ -562,31 +555,19 @@ void
 _mesa_update_renderbuffer_surface(struct gl_context *ctx,
                                   struct gl_renderbuffer *rb)
 {
-   struct pipe_context *pipe = ctx->pipe;
    struct pipe_resource *resource = rb->texture;
    const struct gl_texture_object *stTexObj = NULL;
    unsigned rtt_width = rb->Width;
    unsigned rtt_height = rb->Height;
    unsigned rtt_depth = rb->Depth;
 
-   /*
-    * For winsys fbo, it is possible that the renderbuffer is sRGB-capable but
-    * the format of rb->texture is linear (because we have no control over
-    * the format).  Check rb->Format instead of rb->texture->format
-    * to determine if the rb is sRGB-capable.
-    */
-   bool enable_srgb = ctx->Color.sRGBEnabled &&
-      _mesa_is_format_srgb(rb->Format);
-   enum pipe_format format = resource->format;
+
+   enum pipe_format format = _mesa_renderbuffer_get_format(ctx, rb);
 
    if (rb->is_rtt) {
       stTexObj = rb->TexImage->TexObject;
-      if (stTexObj->surface_based)
-         format = stTexObj->surface_format;
    }
 
-   format = enable_srgb ? util_format_srgb(format) : util_format_linear(format);
-
    if (resource->target == PIPE_TEXTURE_1D_ARRAY) {
       rtt_depth = rtt_height;
       rtt_height = 1;
@@ -646,7 +627,7 @@ _mesa_update_renderbuffer_surface(struct gl_context *ctx,
          than or equal to samples and no more than the next larger sample count 
          supported by the implementation.
        */
-      for (unsigned i = rb->rtt_nr_samples; i <= ctx->Const.MaxFramebufferSamples; i++) {
+      for (unsigned i = rb->rtt_nr_samples + 1; i <= ctx->Const.MaxFramebufferSamples; i++) {
          if (!ctx->st->screen->is_format_supported(ctx->st->screen, format, resource->target, i, i, resource->bind))
             continue;
          nr_samples = i;
@@ -654,32 +635,10 @@ _mesa_update_renderbuffer_surface(struct gl_context *ctx,
       }
    }
 
-   struct pipe_surface **psurf =
-      enable_srgb ? &rb->surface_srgb : &rb->surface_linear;
-   struct pipe_surface *surf = *psurf;
-
-   if (!surf ||
-       surf->texture->nr_samples != rb->NumSamples ||
-       surf->texture->nr_storage_samples != rb->NumStorageSamples ||
-       surf->format != format ||
-       surf->texture != resource ||
-       surf->nr_samples != nr_samples ||
-       surf->u.tex.level != level ||
-       surf->u.tex.first_layer != first_layer ||
-       surf->u.tex.last_layer != last_layer) {
-      /* create a new pipe_surface */
-      struct pipe_surface surf_tmpl;
-      memset(&surf_tmpl, 0, sizeof(surf_tmpl));
-      surf_tmpl.format = format;
-      surf_tmpl.nr_samples = nr_samples;
-      surf_tmpl.u.tex.level = level;
-      surf_tmpl.u.tex.first_layer = first_layer;
-      surf_tmpl.u.tex.last_layer = last_layer;
-
-      /* create -> destroy to avoid blowing up cached surfaces */
-      struct pipe_surface *surf = pipe->create_surface(pipe, resource, &surf_tmpl);
-      pipe_surface_release(pipe, psurf);
-      *psurf = surf;
-   }
-   rb->surface = *psurf;
+   rb->surface.format = format;
+   rb->surface.texture = rb->texture;
+   rb->surface.nr_samples = nr_samples;
+   rb->surface.level = level;
+   rb->surface.first_layer = first_layer;
+   rb->surface.last_layer = last_layer;
 }
