@@ -13,12 +13,14 @@
 #include "panvk_priv_bo.h"
 
 static void
-cmd_write_timestamp(struct cs_builder *b, uint64_t addr)
+cmd_write_timestamp(const struct panvk_device *dev, struct cs_builder *b,
+                    uint64_t addr)
 {
    const struct cs_index addr_reg = cs_scratch_reg64(b, 0);
    /* abuse DEFERRED_SYNC */
-   const struct cs_async_op async = cs_defer(
-      SB_ALL_ITERS_MASK | SB_MASK(DEFERRED_FLUSH), SB_ID(DEFERRED_SYNC));
+   const struct cs_async_op async =
+      cs_defer(dev->csf.sb.all_iters_mask | SB_MASK(DEFERRED_FLUSH),
+               SB_ID(DEFERRED_SYNC));
 
    cs_move64_to(b, addr_reg, addr);
    cs_store_state(b, addr_reg, 0, MALI_CS_STATE_TIMESTAMP, async);
@@ -31,7 +33,7 @@ cmd_copy_data(struct cs_builder *b, uint64_t dst_addr, uint64_t src_addr,
    assert((dst_addr | src_addr | size) % sizeof(uint32_t) == 0);
 
    /* wait for timestamp writes */
-   cs_wait_slot(b, SB_ID(DEFERRED_SYNC), false);
+   cs_wait_slot(b, SB_ID(DEFERRED_SYNC));
 
    /* Depending on where this is called from, we could potentially use SR
     * registers or copy with a compute job.
@@ -52,7 +54,7 @@ cmd_copy_data(struct cs_builder *b, uint64_t dst_addr, uint64_t src_addr,
          const struct cs_index reg = cs_scratch_reg_tuple(b, 4, count);
 
          cs_load_to(b, reg, src_addr_reg, BITFIELD_MASK(count), offset);
-         cs_wait_slot(b, SB_ID(LS), false);
+         cs_wait_slot(b, SB_ID(LS));
          cs_store(b, reg, dst_addr_reg, BITFIELD_MASK(count), offset);
 
          copy_count -= count;
@@ -64,7 +66,7 @@ cmd_copy_data(struct cs_builder *b, uint64_t dst_addr, uint64_t src_addr,
       size -= offset;
    }
 
-   cs_wait_slot(b, SB_ID(LS), false);
+   cs_wait_slot(b, SB_ID(LS));
 }
 
 static struct cs_builder *
@@ -80,20 +82,40 @@ static void
 panvk_utrace_record_ts(struct u_trace *ut, void *cs, void *timestamps,
                        uint64_t offset_B, uint32_t flags)
 {
-   struct cs_builder *b = get_builder(cs, ut);
+   struct panvk_cmd_buffer *cmdbuf = cs;
+   struct panvk_device *dev = to_panvk_device(cmdbuf->vk.base.device);
+   struct cs_builder *b = get_builder(cmdbuf, ut);
    const struct panvk_priv_bo *bo = timestamps;
    const uint64_t addr = bo->addr.dev + offset_B;
 
-   cmd_write_timestamp(b, addr);
+   cmd_write_timestamp(dev, b, addr);
+}
+
+static void
+panvk_utrace_capture_data(struct u_trace *ut, void *cs, void *dst_buffer,
+                          uint64_t dst_offset_B, void *src_buffer,
+                          uint64_t src_offset_B, uint32_t size_B)
+{
+   struct cs_builder *b = get_builder(cs, ut);
+   const struct panvk_priv_bo *dst_bo = dst_buffer;
+   const uint64_t dst_addr = dst_bo->addr.dev + dst_offset_B;
+   const uint64_t src_addr = src_offset_B;
+
+   /* src_offset_B is absolute */
+   assert(!src_buffer);
+
+   cmd_copy_data(b, dst_addr, src_addr, size_B);
 }
 
 void
 panvk_per_arch(utrace_context_init)(struct panvk_device *dev)
 {
-   u_trace_context_init(&dev->utrace.utctx, dev, sizeof(uint64_t), 0,
+   u_trace_context_init(&dev->utrace.utctx, dev, sizeof(uint64_t),
+                        sizeof(VkDispatchIndirectCommand),
                         panvk_utrace_create_buffer, panvk_utrace_delete_buffer,
-                        panvk_utrace_record_ts, panvk_utrace_read_ts, NULL,
-                        NULL, panvk_utrace_delete_flush_data);
+                        panvk_utrace_record_ts, panvk_utrace_read_ts,
+                        panvk_utrace_capture_data, panvk_utrace_get_data,
+                        panvk_utrace_delete_flush_data);
 }
 
 void
@@ -136,8 +158,7 @@ alloc_clone_buffer(void *cookie)
    const uint32_t size = 4 * 1024;
    const uint32_t alignment = 64;
 
-   struct panfrost_ptr ptr =
-      pan_pool_alloc_aligned(&pool->base, size, alignment);
+   struct pan_ptr ptr = pan_pool_alloc_aligned(&pool->base, size, alignment);
 
    return (struct cs_buffer){
       .cpu = ptr.cpu,
@@ -157,6 +178,7 @@ panvk_per_arch(utrace_clone_init_builder)(struct cs_builder *b,
       .nr_kernel_registers = MAX2(csif_info->unpreserved_cs_reg_count, 4),
       .alloc_buffer = alloc_clone_buffer,
       .cookie = pool,
+      .ls_sb_slot = SB_ID(LS),
    };
    cs_builder_init(b, &builder_conf, (struct cs_buffer){0});
 }
@@ -167,9 +189,10 @@ panvk_per_arch(utrace_clone_finish_builder)(struct cs_builder *b)
    const struct cs_index flush_id = cs_scratch_reg32(b, 0);
 
    cs_move32_to(b, flush_id, 0);
-   cs_flush_caches(b, MALI_CS_FLUSH_MODE_CLEAN, MALI_CS_FLUSH_MODE_NONE, false,
-                   flush_id, cs_defer(SB_IMM_MASK, SB_ID(IMM_FLUSH)));
-   cs_wait_slot(b, SB_ID(IMM_FLUSH), false);
+   cs_flush_caches(b, MALI_CS_FLUSH_MODE_CLEAN, MALI_CS_FLUSH_MODE_NONE,
+                   MALI_CS_OTHER_FLUSH_MODE_NONE, flush_id,
+                   cs_defer(SB_IMM_MASK, SB_ID(IMM_FLUSH)));
+   cs_wait_slot(b, SB_ID(IMM_FLUSH));
 
    cs_finish(b);
 }
