@@ -26,6 +26,7 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include "pan_afbc.h"
 #include "pan_props.h"
 
 #include "panvk_device.h"
@@ -45,12 +46,44 @@
 #include "vk_object.h"
 #include "vk_util.h"
 
+bool
+panvk_image_can_use_afbc(
+   struct panvk_physical_device *phys_dev, VkFormat fmt,
+   VkImageUsageFlags usage, VkImageType type, VkImageTiling tiling,
+   VkImageCreateFlags flags)
+{
+   unsigned arch = pan_arch(phys_dev->kmod.props.gpu_id);
+   struct panvk_instance *instance = to_panvk_instance(phys_dev->vk.instance);
+   enum pipe_format pfmt = vk_format_to_pipe_format(fmt);
+
+   /* Disallow AFBC if either of these is true
+    * - PANVK_DEBUG does not have the 'afbc' flag set
+    * - storage image views are requested
+    * - host image copies are requested
+    * - the GPU doesn't support AFBC
+    * - the format is not AFBC-able
+    * - tiling is set to linear
+    * - this is a 1D image
+    * - this is a 3D image on a pre-v7 GPU
+    * - this is a mutable format image on v7
+    */
+   return
+      (instance->debug_flags & PANVK_DEBUG_AFBC) &&
+      !(usage & (VK_IMAGE_USAGE_STORAGE_BIT |
+                 VK_IMAGE_USAGE_HOST_TRANSFER_BIT)) &&
+      pan_query_afbc(&phys_dev->kmod.props) &&
+      pan_afbc_supports_format(arch, pfmt) &&
+      tiling == VK_IMAGE_TILING_OPTIMAL &&
+      type != VK_IMAGE_TYPE_1D &&
+      (type != VK_IMAGE_TYPE_3D || arch >= 7) &&
+      (!(flags & VK_IMAGE_CREATE_MUTABLE_FORMAT_BIT) || arch != 7);
+}
+
 static bool
 panvk_image_can_use_mod(struct panvk_image *image, uint64_t mod)
 {
    struct panvk_physical_device *phys_dev =
       to_panvk_physical_device(image->vk.base.device->physical);
-   unsigned arch = pan_arch(phys_dev->kmod.props.gpu_prod_id);
    struct panvk_instance *instance =
       to_panvk_instance(image->vk.base.device->physical->instance);
    enum pipe_format pfmt = vk_format_to_pipe_format(image->vk.format);
@@ -64,28 +97,17 @@ panvk_image_can_use_mod(struct panvk_image *image, uint64_t mod)
       return mod == DRM_FORMAT_MOD_LINEAR;
 
    if (drm_is_afbc(mod)) {
-      /* Disallow AFBC if either of these is true
-       * - PANVK_DEBUG does not have the 'afbc' flag set
-       * - storage image views are requested
-       * - this is a multisample image
-       * - the GPU doesn't support AFBC
-       * - the format is not AFBC-able
-       * - tiling is set to linear
-       * - this is a 1D image
-       * - this is a 3D image on a pre-v7 GPU
-       * - this is a mutable format image on v7
-       */
-      if (!(instance->debug_flags & PANVK_DEBUG_AFBC) ||
-          ((image->vk.usage | image->vk.stencil_usage) &
-           VK_IMAGE_USAGE_STORAGE_BIT) ||
-          image->vk.samples > 1 ||
-          !panfrost_query_afbc(&phys_dev->kmod.props) ||
-          !panfrost_format_supports_afbc(arch, pfmt) ||
-          image->vk.tiling == VK_IMAGE_TILING_LINEAR ||
-          image->vk.image_type == VK_IMAGE_TYPE_1D ||
-          (image->vk.image_type == VK_IMAGE_TYPE_3D && arch < 7) ||
-          ((image->vk.create_flags & VK_IMAGE_CREATE_MUTABLE_FORMAT_BIT) &&
-           arch == 7))
+      if (!panvk_image_can_use_afbc(
+            phys_dev, image->vk.format,
+            image->vk.usage | image->vk.stencil_usage, image->vk.image_type,
+            image->vk.tiling, image->vk.create_flags))
+         return false;
+
+      /* AFBC is not supported for multisample images
+       *
+       * This is not checked in panvk_image_can_use_afbc because the sample
+       * count is not known in vkGetPhysicalDeviceImageFormatProperties2. */
+      if (image->vk.samples > 1)
          return false;
 
       const struct util_format_description *fdesc =
@@ -96,11 +118,23 @@ panvk_image_can_use_mod(struct panvk_image *image, uint64_t mod)
       if ((mod & AFBC_FORMAT_MOD_YTR) && (!is_rgb || fdesc->nr_channels >= 3))
          return false;
 
+      /* AFBC headers point to their tile with a 32-bit offset, so we can't
+       * have a body size that's bigger than UINT32_MAX. */
+      uint64_t body_size = (uint64_t)image->vk.extent.width *
+                           image->vk.extent.height * image->vk.extent.depth *
+                           util_format_get_blocksize(pfmt);
+      if (body_size > UINT32_MAX)
+         return false;
+
       /* We assume all other unsupported AFBC modes have been filtered out
        * through pan_best_modifiers[]. */
       return true;
    }
 
+   /* Some formats can only be used with AFBC. */
+   if (!pan_u_tiled_or_linear_supports_format(pfmt))
+      return false;
+
    if (mod == DRM_FORMAT_MOD_ARM_16X16_BLOCK_U_INTERLEAVED) {
       /* Multiplanar YUV with U-interleaving isn't supported by the HW. We
        * also need to make sure images that can be aliased to planes of
@@ -148,13 +182,15 @@ static uint64_t
 panvk_image_get_mod_from_list(struct panvk_image *image,
                               const uint64_t *mods, uint32_t mod_count)
 {
-   for (unsigned i = 0; i < PAN_MODIFIER_COUNT; ++i) {
-      if (!panvk_image_can_use_mod(image, pan_best_modifiers[i]))
+   PAN_SUPPORTED_MODIFIERS(supported_mods);
+
+   for (unsigned i = 0; i < ARRAY_SIZE(supported_mods); ++i) {
+      if (!panvk_image_can_use_mod(image, supported_mods[i]))
          continue;
 
       if (!mod_count ||
-          drm_find_modifier(pan_best_modifiers[i], mods, mod_count))
-         return pan_best_modifiers[i];
+          drm_find_modifier(supported_mods[i], mods, mod_count))
+         return supported_mods[i];
    }
 
    /* If we reached that point without finding a proper modifier, there's
@@ -205,13 +241,37 @@ panvk_image_type_to_mali_tex_dim(VkImageType type)
    }
 }
 
-static void
+static bool
+is_disjoint(const struct panvk_image *image)
+{
+   assert((image->plane_count > 1 &&
+           image->vk.format != VK_FORMAT_D32_SFLOAT_S8_UINT) ||
+          (image->vk.create_flags & VK_IMAGE_CREATE_ALIAS_BIT) ||
+          !(image->vk.create_flags & VK_IMAGE_CREATE_DISJOINT_BIT));
+   return image->vk.create_flags & VK_IMAGE_CREATE_DISJOINT_BIT;
+}
+
+static bool
+strict_import(struct panvk_image *image, uint32_t plane)
+{
+   /* We can't do strict imports for AFBC because a Vulkan-based compositor
+    * might be importing buffers from clients that are relying on the old
+    * behavior. The only exception is AFBC(YUV) because support for these
+    * formats was added after we started enforcing WSI pitch. */
+   if (drm_is_afbc(image->vk.drm_format_mod) &&
+       !pan_format_is_yuv(image->planes[plane].image.props.format))
+      return false;
+
+   return true;
+}
+
+static VkResult
 panvk_image_init_layouts(struct panvk_image *image,
                          const VkImageCreateInfo *pCreateInfo)
 {
    struct panvk_physical_device *phys_dev =
       to_panvk_physical_device(image->vk.base.device->physical);
-   unsigned arch = pan_arch(phys_dev->kmod.props.gpu_prod_id);
+   unsigned arch = pan_arch(phys_dev->kmod.props.gpu_id);
    const VkImageDrmFormatModifierExplicitCreateInfoEXT *explicit_info =
       vk_find_struct_const(
          pCreateInfo->pNext,
@@ -226,6 +286,11 @@ panvk_image_init_layouts(struct panvk_image *image,
    if (image->vk.format == VK_FORMAT_D32_SFLOAT_S8_UINT)
       image->plane_count = 2;
 
+   const struct pan_mod_handler *mod_handler =
+      pan_mod_get_handler(arch, image->vk.drm_format_mod);
+   struct pan_image_layout_constraints plane_layout = {
+      .offset_B = 0,
+   };
    for (uint8_t plane = 0; plane < image->plane_count; plane++) {
       VkFormat format;
 
@@ -234,30 +299,45 @@ panvk_image_init_layouts(struct panvk_image *image,
       else
          format = vk_format_get_plane_format(image->vk.format, plane);
 
-      struct pan_image_explicit_layout plane_layout;
-      if (explicit_info)
-         plane_layout = (struct pan_image_explicit_layout){
-            .offset = explicit_info->pPlaneLayouts[plane].offset,
-            .row_stride = explicit_info->pPlaneLayouts[plane].rowPitch,
+      if (explicit_info) {
+         plane_layout = (struct pan_image_layout_constraints){
+            .offset_B = explicit_info->pPlaneLayouts[plane].offset,
+            .wsi_row_pitch_B = explicit_info->pPlaneLayouts[plane].rowPitch,
          };
+      }
 
-      image->planes[plane].layout = (struct pan_image_layout){
-         .format = vk_format_to_pipe_format(format),
-         .dim = panvk_image_type_to_mali_tex_dim(image->vk.image_type),
-         .width = vk_format_get_plane_width(image->vk.format, plane,
-                                            image->vk.extent.width),
-         .height = vk_format_get_plane_height(image->vk.format, plane,
-                                              image->vk.extent.height),
-         .depth = image->vk.extent.depth,
-         .array_size = image->vk.array_layers,
-         .nr_samples = image->vk.samples,
-         .nr_slices = image->vk.mip_levels,
+      image->planes[plane].image = (struct pan_image){
+         .props = {
+            .modifier = image->vk.drm_format_mod,
+            .format = vk_format_to_pipe_format(format),
+            .dim = panvk_image_type_to_mali_tex_dim(image->vk.image_type),
+            .extent_px = {
+               .width = vk_format_get_plane_width(image->vk.format, plane,
+                                                  image->vk.extent.width),
+               .height = vk_format_get_plane_height(image->vk.format, plane,
+                                                    image->vk.extent.height),
+               .depth = image->vk.extent.depth,
+            },
+            .array_size = image->vk.array_layers,
+            .nr_samples = image->vk.samples,
+            .nr_slices = image->vk.mip_levels,
+         },
+         .mod_handler = mod_handler,
+         .planes = {&image->planes[plane].plane},
       };
 
-      image->planes[plane].layout.modifier = image->vk.drm_format_mod;
-      pan_image_layout_init(arch, &image->planes[plane].layout,
-                            explicit_info ? &plane_layout : NULL);
+      plane_layout.strict = strict_import(image, plane);
+      if (!pan_image_layout_init(arch, &image->planes[plane].image, 0,
+                                 &plane_layout)) {
+         return panvk_error(image->vk.base.device,
+                            VK_ERROR_INITIALIZATION_FAILED);
+      }
+
+      if (!is_disjoint(image) && !explicit_info)
+         plane_layout.offset_B += image->planes[plane].plane.layout.data_size_B;
    }
+
+   return VK_SUCCESS;
 }
 
 static void
@@ -320,23 +400,16 @@ static uint64_t
 panvk_image_get_total_size(const struct panvk_image *image)
 {
    uint64_t size = 0;
-   for (uint8_t plane = 0; plane < image->plane_count; plane++)
-      size += image->planes[plane].layout.data_size;
+   for (uint8_t plane = 0; plane < image->plane_count; plane++) {
+      const struct pan_image_layout *layout =
+         &image->planes[plane].plane.layout;
+      size = MAX2(size, layout->slices[0].offset_B + layout->data_size_B);
+   }
    return size;
 }
 
-static bool
-is_disjoint(struct panvk_image *image)
-{
-   assert((image->plane_count > 1 &&
-           image->vk.format != VK_FORMAT_D32_SFLOAT_S8_UINT) ||
-          (image->vk.create_flags & VK_IMAGE_CREATE_ALIAS_BIT) ||
-          !(image->vk.create_flags & VK_IMAGE_CREATE_DISJOINT_BIT));
-   return image->vk.create_flags & VK_IMAGE_CREATE_DISJOINT_BIT;
-}
-
-static void
-panvk_image_init(struct panvk_device *dev, struct panvk_image *image,
+static VkResult
+panvk_image_init(struct panvk_image *image,
                  const VkImageCreateInfo *pCreateInfo)
 {
    /* Add any create/usage flags that might be needed for meta operations.
@@ -347,7 +420,50 @@ panvk_image_init(struct panvk_device *dev, struct panvk_image *image,
    /* Now that we've patched the create/usage flags, we can proceed with the
     * modifier selection. */
    image->vk.drm_format_mod = panvk_image_get_mod(image, pCreateInfo);
-   panvk_image_init_layouts(image, pCreateInfo);
+   return panvk_image_init_layouts(image, pCreateInfo);
+}
+
+static VkResult
+panvk_image_plane_bind(struct panvk_device *dev,
+                       struct panvk_image_plane *plane, struct pan_kmod_bo *bo,
+                       uint64_t base, uint64_t offset)
+{
+   plane->plane.base = base + offset;
+   plane->offset = offset;
+   /* Reset the AFBC headers */
+   if (drm_is_afbc(plane->image.props.modifier)) {
+      /* Transient CPU mapping */
+      void *bo_base = pan_kmod_bo_mmap(bo, 0, pan_kmod_bo_size(bo),
+                                       PROT_WRITE, MAP_SHARED, NULL);
+
+      if (bo_base == MAP_FAILED)
+         return panvk_errorf(dev, VK_ERROR_OUT_OF_HOST_MEMORY,
+                             "Failed to CPU map AFBC image plane");
+
+      for (unsigned layer = 0; layer < plane->image.props.array_size;
+           layer++) {
+         for (unsigned level = 0; level < plane->image.props.nr_slices;
+              level++) {
+            const struct pan_image_slice_layout *slayout =
+               &plane->plane.layout.slices[level];
+            uint32_t z_slice_count =
+               u_minify(plane->image.props.extent_px.depth, level);
+
+            for (unsigned z = 0; z < z_slice_count; z++) {
+               void *header = bo_base + offset +
+                              ((uint64_t)slayout->afbc.surface_stride_B * z) +
+                              (layer * plane->plane.layout.array_stride_B) +
+                              plane->plane.layout.slices[level].offset_B;
+               memset(header, 0, slayout->afbc.header.surface_size_B);
+            }
+         }
+      }
+
+      ASSERTED int ret = os_munmap(bo_base, pan_kmod_bo_size(bo));
+      assert(!ret);
+   }
+
+   return VK_SUCCESS;
 }
 
 VKAPI_ATTR VkResult VKAPI_CALL
@@ -372,7 +488,11 @@ panvk_CreateImage(VkDevice device, const VkImageCreateInfo *pCreateInfo,
    if (!image)
       return panvk_error(device, VK_ERROR_OUT_OF_HOST_MEMORY);
 
-   panvk_image_init(dev, image, pCreateInfo);
+   VkResult result = panvk_image_init(image, pCreateInfo);
+   if (result != VK_SUCCESS) {
+      vk_image_destroy(&dev->vk, pAllocator, &image->vk);
+      return result;
+   }
 
    /*
     * From the Vulkan spec:
@@ -399,39 +519,85 @@ panvk_DestroyImage(VkDevice _device, VkImage _image,
    if (!image)
       return;
 
-   if (image->bo)
-      pan_kmod_bo_put(image->bo);
-
    vk_image_destroy(&device->vk, pAllocator, &image->vk);
 }
 
-VKAPI_ATTR void VKAPI_CALL
-panvk_GetImageSubresourceLayout(VkDevice _device, VkImage _image,
-                                const VkImageSubresource *pSubresource,
-                                VkSubresourceLayout *pLayout)
+static void
+get_image_subresource_layout(const struct panvk_image *image,
+                             const VkImageSubresource2 *subres2,
+                             VkSubresourceLayout2 *layout2)
 {
-   VK_FROM_HANDLE(panvk_image, image, _image);
-
-   unsigned plane =
-      panvk_plane_index(image->vk.format, pSubresource->aspectMask);
+   const VkImageSubresource *subres = &subres2->imageSubresource;
+   VkSubresourceLayout *layout = &layout2->subresourceLayout;
+   unsigned plane = panvk_plane_index(image->vk.format, subres->aspectMask);
    assert(plane < PANVK_MAX_PLANES);
 
    const struct pan_image_slice_layout *slice_layout =
-      &image->planes[plane].layout.slices[pSubresource->mipLevel];
+      &image->planes[plane].plane.layout.slices[subres->mipLevel];
 
-   uint64_t base_offset = 0;
-   if (!is_disjoint(image)) {
-      for (uint8_t plane_idx = 0; plane_idx < plane; plane_idx++)
-         base_offset += image->planes[plane_idx].layout.data_size;
+   layout->offset =
+      slice_layout->offset_B +
+      (subres->arrayLayer * image->planes[plane].plane.layout.array_stride_B);
+   layout->size = slice_layout->size_B;
+   layout->arrayPitch = image->planes[plane].plane.layout.array_stride_B;
+
+   if (drm_is_afbc(image->vk.drm_format_mod)) {
+      /* row/depth pitch expressed in AFBC superblocks. */
+      layout->rowPitch = pan_afbc_stride_blocks(
+         image->vk.drm_format_mod, slice_layout->afbc.header.row_stride_B);
+      layout->depthPitch = pan_afbc_stride_blocks(
+         image->vk.drm_format_mod, slice_layout->afbc.header.surface_size_B);
+   } else {
+      layout->rowPitch = slice_layout->tiled_or_linear.row_stride_B;
+      layout->depthPitch = slice_layout->tiled_or_linear.surface_stride_B;
    }
 
-   pLayout->offset = base_offset +
-      slice_layout->offset + (pSubresource->arrayLayer *
-                              image->planes[plane].layout.array_stride);
-   pLayout->size = slice_layout->size;
-   pLayout->rowPitch = slice_layout->row_stride;
-   pLayout->arrayPitch = image->planes[plane].layout.array_stride;
-   pLayout->depthPitch = slice_layout->surface_stride;
+   VkSubresourceHostMemcpySize *memcpy_size =
+      vk_find_struct(layout2->pNext, SUBRESOURCE_HOST_MEMCPY_SIZE);
+   if (memcpy_size) {
+      /* When copying to/from a D24S8 image, we can't use the normal memcpy
+       * path because we need to interleave the depth/stencil components. For
+       * the stencil aspect, the copied data only needs 1 byte/px instead of 4.
+       */
+      if (image->vk.format == VK_FORMAT_D24_UNORM_S8_UINT) {
+         switch (subres->aspectMask) {
+            case VK_IMAGE_ASPECT_DEPTH_BIT:
+               memcpy_size->size = slice_layout->size_B;
+               break;
+            case VK_IMAGE_ASPECT_STENCIL_BIT:
+               memcpy_size->size = slice_layout->size_B / 4;
+               break;
+            default:
+               unreachable("invalid aspect");
+         }
+      } else {
+         memcpy_size->size = slice_layout->size_B;
+      }
+   }
+}
+
+VKAPI_ATTR void VKAPI_CALL
+panvk_GetImageSubresourceLayout2(VkDevice device, VkImage image,
+                                 const VkImageSubresource2 *pSubresource,
+                                 VkSubresourceLayout2 *pLayout)
+{
+   VK_FROM_HANDLE(panvk_image, img, image);
+
+   get_image_subresource_layout(img, pSubresource, pLayout);
+}
+
+VKAPI_ATTR void VKAPI_CALL
+panvk_GetDeviceImageSubresourceLayoutKHR(
+   VkDevice device, const VkDeviceImageSubresourceInfoKHR *pInfo,
+   VkSubresourceLayout2KHR *pLayout)
+{
+   VK_FROM_HANDLE(panvk_device, dev, device);
+   struct panvk_image image = {0};
+
+   vk_image_init(&dev->vk, &image.vk, pInfo->pCreateInfo);
+   panvk_image_init(&image, pInfo->pCreateInfo);
+   get_image_subresource_layout(&image, pInfo->pSubresource, pLayout);
+   vk_image_finish(&image.vk);
 }
 
 VKAPI_ATTR void VKAPI_CALL
@@ -449,7 +615,7 @@ panvk_GetImageMemoryRequirements2(VkDevice device,
       plane_info ? plane_info->planeAspect : image->vk.aspects;
    uint8_t plane = panvk_plane_index(image->vk.format, aspects);
    const uint64_t size =
-      disjoint ? image->planes[plane].layout.data_size :
+      disjoint ? image->planes[plane].plane.layout.data_size_B :
       panvk_image_get_total_size(image);
 
    pMemoryRequirements->memoryRequirements.memoryTypeBits = 1;
@@ -480,13 +646,14 @@ panvk_GetDeviceImageMemoryRequirements(VkDevice device,
 
    struct panvk_image image;
    vk_image_init(&dev->vk, &image.vk, pInfo->pCreateInfo);
-   panvk_image_init(dev, &image, pInfo->pCreateInfo);
+   panvk_image_init(&image, pInfo->pCreateInfo);
 
    VkImageMemoryRequirementsInfo2 info2 = {
       .sType = VK_STRUCTURE_TYPE_IMAGE_MEMORY_REQUIREMENTS_INFO_2,
       .image = panvk_image_to_handle(&image),
    };
    panvk_GetImageMemoryRequirements2(device, &info2, pMemoryRequirements);
+   vk_image_finish(&image.vk);
 }
 
 VKAPI_ATTR void VKAPI_CALL
@@ -509,84 +676,65 @@ panvk_GetDeviceImageSparseMemoryRequirements(VkDevice device,
    *pSparseMemoryRequirementCount = 0;
 }
 
-static void
-panvk_image_plane_bind(struct pan_image *plane, struct pan_kmod_bo *bo,
-                       uint64_t base, uint64_t offset)
-{
-   plane->data.base = base;
-   plane->data.offset = offset;
-   /* Reset the AFBC headers */
-   if (drm_is_afbc(plane->layout.modifier)) {
-      /* Transient CPU mapping */
-      void *bo_base = pan_kmod_bo_mmap(bo, 0, pan_kmod_bo_size(bo),
-                                       PROT_WRITE, MAP_SHARED, NULL);
+static VkResult
+panvk_image_bind(struct panvk_device *dev,
+                 const VkBindImageMemoryInfo *bind_info) {
+   VK_FROM_HANDLE(panvk_image, image, bind_info->image);
+   VK_FROM_HANDLE(panvk_device_memory, mem, bind_info->memory);
 
-      assert(bo_base != MAP_FAILED);
-
-      for (unsigned layer = 0; layer < plane->layout.array_size;
-           layer++) {
-         for (unsigned level = 0; level < plane->layout.nr_slices;
-              level++) {
-            void *header = bo_base + plane->data.offset +
-                           (layer * plane->layout.array_stride) +
-                           plane->layout.slices[level].offset;
-            memset(header, 0,
-                   plane->layout.slices[level].afbc.header_size);
-         }
-      }
-
-      ASSERTED int ret = os_munmap(bo_base, pan_kmod_bo_size(bo));
-      assert(!ret);
+   if (!mem) {
+#if DETECT_OS_ANDROID
+      /* TODO handle VkNativeBufferANDROID when we support ANB */
+      unreachable("VkBindImageMemoryInfo with no memory");
+#else
+      const VkBindImageMemorySwapchainInfoKHR *swapchain_info =
+         vk_find_struct_const(bind_info->pNext,
+                              BIND_IMAGE_MEMORY_SWAPCHAIN_INFO_KHR);
+      assert(swapchain_info && swapchain_info->swapchain != VK_NULL_HANDLE);
+      VkDeviceMemory mem_handle = wsi_common_get_memory(
+         swapchain_info->swapchain, swapchain_info->imageIndex);
+      mem = panvk_device_memory_from_handle(mem_handle);
+#endif
    }
+
+   assert(mem);
+   image->mem = mem;
+   if (is_disjoint(image)) {
+      const VkBindImagePlaneMemoryInfo *plane_info =
+         vk_find_struct_const(bind_info->pNext, BIND_IMAGE_PLANE_MEMORY_INFO);
+      const uint8_t plane =
+         panvk_plane_index(image->vk.format, plane_info->planeAspect);
+      return panvk_image_plane_bind(dev, &image->planes[plane], mem->bo,
+                                    mem->addr.dev, bind_info->memoryOffset);
+   } else {
+      for (unsigned plane = 0; plane < image->plane_count; plane++) {
+         VkResult result =
+            panvk_image_plane_bind(dev, &image->planes[plane], mem->bo,
+                                   mem->addr.dev, bind_info->memoryOffset);
+         if (result != VK_SUCCESS)
+            return result;
+      }
+   }
+
+   return VK_SUCCESS;
 }
 
 VKAPI_ATTR VkResult VKAPI_CALL
 panvk_BindImageMemory2(VkDevice device, uint32_t bindInfoCount,
                        const VkBindImageMemoryInfo *pBindInfos)
 {
-   const VkBindImageMemorySwapchainInfoKHR *swapchain_info =
-      vk_find_struct_const(pBindInfos->pNext, BIND_IMAGE_MEMORY_SWAPCHAIN_INFO_KHR);
+   VK_FROM_HANDLE(panvk_device, dev, device);
+   VkResult result = VK_SUCCESS;
 
-   for (uint32_t i = 0; i < bindInfoCount; ++i) {
-      VK_FROM_HANDLE(panvk_image, image, pBindInfos[i].image);
-      struct pan_kmod_bo *old_bo = image->bo;
-
-      if (swapchain_info && swapchain_info->swapchain != VK_NULL_HANDLE) {
-         VkImage wsi_vk_image = wsi_common_get_image(swapchain_info->swapchain,
-                                                   swapchain_info->imageIndex);
-         VK_FROM_HANDLE(panvk_image, wsi_image, wsi_vk_image);
-
-         assert(image->plane_count == 1);
-         assert(wsi_image->plane_count == 1);
-
-         image->bo = pan_kmod_bo_get(wsi_image->bo);
-         panvk_image_plane_bind(&image->planes[0], image->bo,
-                                wsi_image->planes[0].data.base,
-                                wsi_image->planes[0].data.offset);
-      } else {
-         VK_FROM_HANDLE(panvk_device_memory, mem, pBindInfos[i].memory);
-         assert(mem);
-         image->bo = pan_kmod_bo_get(mem->bo);
-         uint64_t offset = pBindInfos[i].memoryOffset;
-         if (is_disjoint(image)) {
-            const VkBindImagePlaneMemoryInfo *plane_info =
-               vk_find_struct_const(pBindInfos[i].pNext,
-                                    BIND_IMAGE_PLANE_MEMORY_INFO);
-            uint8_t plane =
-               panvk_plane_index(image->vk.format, plane_info->planeAspect);
-            panvk_image_plane_bind(&image->planes[plane], image->bo,
-                                   mem->addr.dev, offset);
-         } else {
-            for (unsigned plane = 0; plane < image->plane_count; plane++) {
-               panvk_image_plane_bind(&image->planes[plane], image->bo,
-                                      mem->addr.dev, offset);
-               offset += image->planes[plane].layout.data_size;
-            }
-         }
-      }
-
-      pan_kmod_bo_put(old_bo);
+   for (uint32_t i = 0; i < bindInfoCount; i++) {
+      const VkBindMemoryStatus *bind_status =
+         vk_find_struct_const(&pBindInfos[i], BIND_MEMORY_STATUS);
+      VkResult bind_result = panvk_image_bind(dev, &pBindInfos[i]);
+      if (bind_status)
+         *bind_status->pResult = bind_result;
+      if (bind_result != VK_SUCCESS)
+         result = bind_result;
    }
 
-   return VK_SUCCESS;
+   return result;
 }
