@@ -793,6 +793,16 @@ u_trace_init(struct u_trace *ut, struct u_trace_context *utctx)
    list_inithead(&ut->trace_chunks);
 }
 
+void
+u_trace_move(struct u_trace *dst, struct u_trace *src)
+{
+   dst->utctx = src->utctx;
+   list_replace(&src->trace_chunks, &dst->trace_chunks);
+   dst->num_traces = src->num_traces;
+   src->num_traces = 0;
+   list_delinit(&src->trace_chunks);
+}
+
 void
 u_trace_fini(struct u_trace *ut)
 {
@@ -980,13 +990,11 @@ u_trace_appendv(struct u_trace *ut,
                                tp->flags);
 
    if (ut->utctx->enabled_traces & U_TRACE_TYPE_INDIRECTS) {
-      uint64_t dst_offset = 0;
       for (unsigned i = 0; i < n_indirects; i++) {
-         ut->utctx->capture_data(
-            ut, cs, chunk->indirects,
-            ut->utctx->max_indirect_size_bytes * tp_idx + dst_offset,
-            addresses[i].bo, addresses[i].offset, indirect_sizes_B[i]);
-         dst_offset += indirect_sizes_B[i];
+         ut->utctx->capture_data(ut, cs, chunk->indirects,
+                                 ut->utctx->max_indirect_size_bytes * tp_idx,
+                                 addresses[i].bo, addresses[i].offset,
+                                 indirect_sizes_B[i]);
       }
       chunk->has_indirect |= n_indirects > 0;
    }
